@@ -1,19 +1,27 @@
+import json
 import os
 
 from flask import Blueprint, Response, request, stream_with_context, current_app
 
 import cirrocumulus.data_processing as data_processing
 from .dataset_api import DatasetAPI
-from .envir import CIRRO_SERVE, CIRRO_FOOTER, CIRRO_UPLOAD, CIRRO_BRAND
-from .file_system_adapter import get_scheme
+from .envir import CIRRO_SERVE, CIRRO_FOOTER, CIRRO_UPLOAD, CIRRO_BRAND, CIRRO_EMAIL, CIRRO_AUTH, CIRRO_DATABASE, \
+    CIRRO_DATASET_SELECTOR_COLUMNS, CIRRO_CELL_ONTOLOGY, CIRRO_STATIC_DIR, CIRRO_MOUNT, CIRRO_MIXPANEL
 from .invalid_usage import InvalidUsage
 from .job_api import submit_job
-from .util import to_json
+from .util import json_response, get_scheme, get_fs
 
 blueprint = Blueprint('blueprint', __name__)
 
 dataset_api = DatasetAPI()
 
+remapped_urls = dict()
+if os.environ.get(CIRRO_MOUNT) is not None:
+    tokens = os.environ.get(CIRRO_MOUNT).split(',')
+    for token in tokens:
+        bucket, local_path = token.split(':')
+        remapped_urls[bucket] = local_path
+
 
 @blueprint.errorhandler(InvalidUsage)
 def handle_invalid_usage(error):
@@ -21,59 +29,64 @@ def handle_invalid_usage(error):
 
 
 def get_database():
-    return current_app.config['DATABASE']
+    return current_app.config[CIRRO_DATABASE]
 
 
 def get_auth():
-    return current_app.config['AUTH']
+    return current_app.config[CIRRO_AUTH]
 
 
 @blueprint.route('/server', methods=['GET'])
 def handle_server():
     # no login required
-    server = get_database().server()
-    server['clientId'] = get_auth().client_id
+    d = {}
+    d['email'] = os.environ.get(CIRRO_EMAIL)
+    d['capabilities'] = get_database().capabilities()
+    d['clientId'] = get_auth().client_id
+    if os.environ.get(CIRRO_MIXPANEL) is not None:
+        d['mixpanel'] = os.environ[CIRRO_MIXPANEL]
+    if os.environ.get(CIRRO_DATASET_SELECTOR_COLUMNS) is not None:
+        if os.path.exists(os.environ[CIRRO_DATASET_SELECTOR_COLUMNS]):
+            with open(os.environ[CIRRO_DATASET_SELECTOR_COLUMNS], 'rt') as f:
+                d['datasetSelectorColumns'] = json.loads(f.read())
+        else:
+            d['datasetSelectorColumns'] = json.loads(os.environ[CIRRO_DATASET_SELECTOR_COLUMNS])
     if os.environ.get(CIRRO_FOOTER) is not None:
         with open(os.environ.get(CIRRO_FOOTER), 'rt') as f:
-            server['footer'] = f.read()
+            d['footer'] = f.read()
     if os.environ.get(CIRRO_BRAND) is not None:
         with open(os.environ.get(CIRRO_BRAND), 'rt') as f:
-            server['brand'] = f.read()
-    # server['brand'] = os.environ.get(CIRRO_BRAND)
-    server['jobs'] = os.environ.get('GAE_APPLICATION') is None
-    server['upload'] = os.environ.get(CIRRO_UPLOAD) is not None
-    return to_json(server)
+            d['brand'] = f.read()
+    d['upload'] = os.environ.get(CIRRO_UPLOAD) is not None
+    if os.environ.get(CIRRO_CELL_ONTOLOGY) is not None:
+        if os.path.exists(os.environ[CIRRO_CELL_ONTOLOGY]):
+            def parse_term(f):
+                term = dict()
+                for term_line in f:
+                    term_line = term_line.strip()
+                    if term_line.startswith("["):
+                        break
+                    index = term_line.find(':')
+                    if index != -1:
+                        key = term_line[:index]
+                        term[key] = term_line[index + 1:].strip()
+                # if term.get('is_obsolete', '') == 'true':
+                #     return None
+                return term
 
+            terms = []
+            with open(os.environ[CIRRO_CELL_ONTOLOGY], 'rt') as f:  # obo file
+                for line in f:
+                    line = line.strip()
+                    if line == "[Term]":
+                        t = parse_term(f)
+                        if t is not None:
+                            terms.append(t)
 
-@blueprint.route('/filters', methods=['GET'])
-def handle_dataset_filters():
-    """List filters available for a dataset.
-    """
-    email = get_auth().auth()['email']
-    dataset_id = request.args.get('id', '')
-
-    if dataset_id == '':
-        return 'Please provide an id', 400
-
-    dataset_filters = get_database().dataset_filters(email, dataset_id)
-    # {'id': result.id, 'name': result['name']}
-    return to_json(dataset_filters)
-
-
-@blueprint.route('/export_filters', methods=['GET'])
-def handle_export_dataset_filters():
-    """Download filters in a csv file for a dataset.
-    """
-    email = get_auth().auth()['email']
-    dataset_id = request.args.get('id', '')
-
-    if dataset_id == '':
-        return 'Please provide an id', 400
-    database_api = get_database()
-    dataset_filters = database_api.dataset_filters(email, dataset_id)
-    dataset = database_api.get_dataset(email, dataset_id)
-    text = data_processing.handle_export_dataset_filters(dataset_api, dataset, dataset_filters)
-    return Response(text, mimetype='text/plain')
+            d['ontology'] = dict(cellTypes=terms)
+        else:
+            d['ontology'] = dict(cellTypes=os.environ[CIRRO_CELL_ONTOLOGY])
+    return json_response(d)
 
 
 @blueprint.route('/category_name', methods=['GET', 'PUT'])
@@ -87,19 +100,28 @@ def handle_category_name():
         dataset_id = request.args.get('id', '')
         if dataset_id == '':
             return 'Please provide an id', 400
-        return to_json(database_api.category_names(email, dataset_id))
-    content = request.get_json(force=True, cache=False)
-    category = content.get('c')
-    original_name = content.get('o')
-    new_name = content.get('n')
-    dataset_id = content.get('id')
+        return json_response(database_api.category_names(email, dataset_id))
+
     if request.method == 'PUT':
+        content = request.get_json(force=True, cache=False)
+        category = content['name']
+        dataset_id = content['id']
+        original_value = content.get('originalValue')
+        update = {}
+        if 'newValue' in content:
+            update['newValue'] = content['newValue']
+        if 'positiveMarkers' in content:
+            update['positiveMarkers'] = content['positiveMarkers']
+        if 'negativeMarkers' in content:
+            update['negativeMarkers'] = content['negativeMarkers']
+        if 'color' in content:
+            update['color'] = content['color']
         database_api.upsert_category_name(
             email=email,
             category=category,
             dataset_id=dataset_id,
-            original_name=original_name,
-            new_name=new_name)
+            original_value=original_value,
+            update=update)
         return '', 200
 
 
@@ -115,7 +137,7 @@ def handle_feature_set():
     #     dataset_id = request.args.get('ds_id', '')
     #     if filter_id == '' or dataset_id == '':
     #         return 'Please provide an id', 400
-    #     return to_json(database_api.get_dataset_filter(email, dataset_id=dataset_id, filter_id=filter_id))
+    #     return json_response(database_api.get_dataset_filter(email, dataset_id=dataset_id, filter_id=filter_id))
     content = request.get_json(force=True, cache=False)
     set_id = content.get('id')
     dataset_id = content.get('ds_id')
@@ -136,48 +158,55 @@ def handle_feature_set():
             name=name,
             category=category,
             features=features)
-        return to_json({'id': set_id})
+        return json_response({'id': set_id})
     elif request.method == 'DELETE':
         database_api.delete_feature_set(email, dataset_id=dataset_id, set_id=set_id)
-        return to_json('', 204)
+        return json_response('', 204)
 
 
-@blueprint.route('/filter', methods=['GET', 'POST', 'PUT', 'DELETE'])
-def handle_dataset_filter():
-    """CRUD for a dataset filter.
+@blueprint.route('/views', methods=['GET'])
+def handle_dataset_views():
+    """List available views for a dataset.
     """
+    email = get_auth().auth()['email']
+    dataset_id = request.args.get('id', '')
 
+    if dataset_id == '':
+        return 'Please provide an id', 400
+
+    return json_response(get_database().dataset_views(email, dataset_id))
+
+
+@blueprint.route('/view', methods=['GET', 'POST', 'PUT', 'DELETE'])
+def handle_dataset_view():
+    """CRUD for a dataset view.
+    """
     email = get_auth().auth()['email']
     database_api = get_database()
     if request.method == 'GET':
-        filter_id = request.args.get('id', '')
-        dataset_id = request.args.get('ds_id', '')
-        if filter_id == '' or dataset_id == '':
+        view_id = request.args.get('id', '')
+        if view_id == '':
             return 'Please provide an id', 400
-        return to_json(database_api.get_dataset_filter(email, dataset_id=dataset_id, filter_id=filter_id))
-    content = request.get_json(force=True, cache=False)
-    filter_id = content.get('id')
-    dataset_id = content.get('ds_id')
+        return json_response(database_api.get_dataset_view(email, view_id=view_id))
+    d = request.get_json(force=True, cache=False)
+
     # POST=new, PUT=update , DELETE=delete, GET=get
     if request.method == 'PUT' or request.method == 'POST':
-        if request.method == 'PUT' and filter_id is None:
+        dataset_id = d.pop('ds_id')
+        view_id = d.pop('id') if request.method == 'PUT' else None
+        if request.method == 'PUT' and view_id is None:
             return 'Please supply an id', 400
         if request.method == 'POST' and dataset_id is None:
             return 'Please supply a ds_id', 400
-        dataset_filter = content.get('value')
-        filter_name = content.get('name')
-        filter_notes = content.get('notes')
-        filter_id = database_api.upsert_dataset_filter(
+
+        view_id = database_api.upsert_dataset_view(
             email=email,
             dataset_id=dataset_id,
-            filter_id=filter_id if request.method == 'PUT' else None,
-            dataset_filter=dataset_filter,
-            filter_name=filter_name,
-            filter_notes=filter_notes)
-        return to_json({'id': filter_id})
+            view=d)
+        return json_response({'id': view_id})
     elif request.method == 'DELETE':
-        database_api.delete_dataset_filter(email, dataset_id=dataset_id, filter_id=filter_id)
-        return to_json('', 204)
+        database_api.delete_dataset_view(email, view_id=d.pop('id'))
+        return json_response('', 204)
 
 
 @blueprint.route('/schema', methods=['GET'])
@@ -188,14 +217,14 @@ def handle_schema():
     database_api = get_database()
     dataset_id = request.args.get('id', '')
     dataset = database_api.get_dataset(email, dataset_id)
-    schema = dataset_api.schema(dataset)
+    schema = dataset_api.get_schema(dataset)
     schema.update(dataset)  # add title, etc from database to schema
     schema['markers'] = database_api.get_feature_sets(email=email, dataset_id=dataset_id)
-    return to_json(schema)
+    return json_response(schema)
 
 
 def get_file_path(file, dataset_url):
-    # when serving dataset, image must be relative to dataset directory
+    # when serving dataset, file must be relative to dataset directory
     if os.environ.get(CIRRO_SERVE) == 'true':
         _, ext = os.path.splitext(dataset_url)
         if file[0] == '/' or file.find('..') != -1:
@@ -216,7 +245,7 @@ def send_file(file_path):
     #     bytes = f.read()
     # return Response(bytes, mimetype=mimetype[0])
     chunk_size = 4096
-    f = dataset_api.fs_adapter.get_fs(file_path).open(file_path)
+    f = get_fs(file_path).open(file_path)
 
     def generate():
         while True:
@@ -236,8 +265,19 @@ def handle_file():
     email = get_auth().auth()['email']
     database_api = get_database()
     dataset_id = request.args.get('id', '')
-    dataset = database_api.get_dataset(email, dataset_id)
-    file_path = get_file_path(request.args.get('file'), dataset['url'])
+    url = request.args.get('file')
+    if dataset_id == '':  # allow if file is in static directory
+        static_dirs = os.environ.get(CIRRO_STATIC_DIR)
+        if static_dirs is None:
+            return 'Not authorized', 401
+        static_dirs = static_dirs.split(',')
+        if os.path.dirname(url) in static_dirs:
+            return send_file(url)
+        return 'Not authorized', 401
+    else:
+        dataset = database_api.get_dataset(email, dataset_id)
+        file_path = get_file_path(url, dataset['url'])
+        file_path = map_url(file_path)
     return send_file(file_path)
 
 
@@ -246,7 +286,16 @@ def handle_user():
     email = get_auth().auth()['email']
     database_api = get_database()
     user = database_api.user(email)
-    return to_json(user)
+    return json_response(user)
+
+
+def map_url(url):
+    for remote_path in remapped_urls:
+        before_replace = url
+        url = url.replace(remote_path, remapped_urls[remote_path])
+        if before_replace != url:  # only replace URL once
+            return url
+    return url
 
 
 def get_email_and_dataset(content):
@@ -256,6 +305,7 @@ def get_email_and_dataset(content):
         return 'Please supply an id', 400
     database_api = get_database()
     dataset = database_api.get_dataset(email, dataset_id)
+    dataset['url'] = map_url(dataset['url'])
     return email, dataset
 
 
@@ -263,14 +313,14 @@ def get_email_and_dataset(content):
 def handle_data():
     json_request = request.get_json(cache=False)
     email, dataset = get_email_and_dataset(json_request)
-
-    return to_json(
-        data_processing.handle_data(dataset_api=dataset_api, dataset=dataset,
-            embedding_list=json_request.get('embedding'),
-            values=json_request.get('values'),
-            stats=json_request.get('stats'),
-            grouped_stats=json_request.get('groupedStats'),
-            selection=json_request.get('selection')))
+    return json_response(
+        data_processing.handle_data(dataset_api=dataset_api,
+                                    dataset=dataset,
+                                    embedding_list=json_request.get('embedding'),
+                                    values=json_request.get('values'),
+                                    stats=json_request.get('stats'),
+                                    grouped_stats=json_request.get('groupedStats'),
+                                    selection=json_request.get('selection')))
 
 
 @blueprint.route('/selection', methods=['POST'])
@@ -279,9 +329,11 @@ def handle_selection():
     content = request.get_json(cache=False)
     email, dataset = get_email_and_dataset(content)
     data_filter = content.get('filter')
-    return to_json(data_processing.handle_selection(dataset_api=dataset_api, dataset=dataset, data_filter=data_filter,
-        measures=content.get('measures', []), dimensions=content.get('dimensions', []),
-        embeddings=content.get('embeddings', [])))
+    return json_response(
+        data_processing.handle_selection(dataset_api=dataset_api, dataset=dataset, data_filter=data_filter,
+                                         measures=content.get('measures', []),
+                                         dimensions=content.get('dimensions', []),
+                                         embeddings=content.get('embeddings', [])))
 
 
 @blueprint.route('/selected_ids', methods=['POST'])
@@ -289,7 +341,7 @@ def handle_selected_ids():
     content = request.get_json(cache=False)
     email, dataset = get_email_and_dataset(content)
     data_filter = content.get('filter')
-    return to_json(
+    return json_response(
         data_processing.handle_selection_ids(dataset_api=dataset_api, dataset=dataset, data_filter=data_filter))
 
 
@@ -298,35 +350,34 @@ def handle_selected_ids():
 def handle_datasets():
     email = get_auth().auth()['email']
     database_api = get_database()
-    return to_json(database_api.datasets(email))
+    return json_response(database_api.datasets(email))
 
 
 def upload_file(file):
     from werkzeug.utils import secure_filename
     filename = secure_filename(file.filename)
     dest = os.path.join(os.environ.get(CIRRO_UPLOAD), filename)
-    dest_fs = dataset_api.fs_adapter.get_fs(dest).open(dest, mode='wb')
+    dest_fs = get_fs(dest).open(dest, mode='wb')
     file.save(dest_fs)
     dest_fs.close()
     return dest
 
 
 def copy_url(url):
-    import fsspec
     from werkzeug.utils import secure_filename
     upload = os.environ.get(CIRRO_UPLOAD)
 
-    if url.endswith('/'):  # don't copy directories
+    if url.endswith('/') or os.path.isdir(url):  # don't copy directories
         return url
     from urllib.parse import urlparse
     if urlparse(upload).netloc == urlparse(url).netloc:  # don't copy if already in the same bucket
         return url
     src_scheme = get_scheme(url)
-    src_fs = fsspec.filesystem(src_scheme)
+    src_fs = get_fs(src_scheme)
     filename = secure_filename(os.path.basename(url))
     dest = os.path.join(upload, filename)
     dest_scheme = get_scheme(dest)
-    dest_fs = fsspec.filesystem(dest_scheme)
+    dest_fs = get_fs(dest_scheme)
     out = dest_fs.open(dest, 'wb')
     n = 1024 * 1024
     with src_fs.open(url, mode='rb') as r:
@@ -345,15 +396,21 @@ def handle_dataset():
     database_api = get_database()
     # POST=new dataset, PUT=update dataset, DELETE=delete, GET=get dataset info
     if request.method == 'PUT' or request.method == 'POST':
-        dataset_id = request.form.get('id')
-        dataset_name = request.form.get('name')
-        description = request.form.get('description')
-        species = request.form.get('species')
-        title = request.form.get('title')
-        url = request.form.get('url')  # e.g. gs://foo/a/b/
-        file = request.files.get('file')
-        readers = request.form.get('readers')
-        if readers is not None:
+        if request.content_type == 'application/json':
+            d = request.get_json(force=True, cache=False)
+        else:
+            d = dict()
+            for key in request.form:
+                d[key] = request.form[key]
+
+        dataset_id = d.get('id')
+        dataset_name = d.get('name')
+        url = d.get('url')  # e.g. gs://foo/a/b/
+        readers = d.pop('readers') if 'readers' in d else None
+        file = None
+        if request.content_type != 'application/json':
+            file = request.files.get('file')  # file upload
+        if readers is not None and not isinstance(readers, list):
             import json
             readers = json.loads(readers)
         if request.method == 'PUT' and dataset_id is None:  # update
@@ -363,23 +420,42 @@ def handle_dataset():
 
         if url is not None and os.environ.get(CIRRO_UPLOAD) is not None:
             url = copy_url(url)
+            d['url'] = url
 
         if file is not None:
             if os.environ.get(CIRRO_UPLOAD) is None:
                 return 'Upload not supported', 400
             url = upload_file(file)
-        dataset_id = database_api.upsert_dataset(email=email,
-            dataset_id=dataset_id if request.method == 'PUT' else None,
-            dataset_name=dataset_name, url=url, readers=readers, description=description, title=title, species=species)
-        return to_json({'id': dataset_id})
+            d['url'] = url
+        if request.method == 'POST' and url is None:  # new
+            return 'Must supply dataset URL', 400
+        dataset_id = database_api.upsert_dataset(email=email, readers=readers, dataset=d)
+        return json_response({'id': dataset_id})
     elif request.method == 'DELETE':
         content = request.get_json(force=True, cache=False)
         dataset_id = content.get('id', '')
         database_api.delete_dataset(email, dataset_id)
-        return to_json('', 204)
+        return json_response('', 204)
     elif request.method == 'GET':
         dataset_id = request.args.get('id', '')
-        return to_json(database_api.get_dataset(email, dataset_id, True))
+        return json_response(database_api.get_dataset(email, dataset_id, True))
+
+
+@blueprint.route('/job_status', methods=['GET'])
+def handle_job_status():
+    email = get_auth().auth()['email']
+    database_api = get_database()
+    job_id = request.args.get('id', '')
+    return database_api.get_job(email=email, job_id=job_id, return_result=False)
+
+
+@blueprint.route('/module', methods=['POST'])
+def handle_module_score():
+    content = request.get_json(cache=False)
+    email, dataset = get_email_and_dataset(content)
+    features = content.get('features')
+    adata = dataset_api.read_dataset(dataset, keys=dict(X=features))
+    return json_response(adata.X.mean(axis=1))
 
 
 @blueprint.route('/job', methods=['GET', 'DELETE'])
@@ -390,22 +466,43 @@ def handle_job():
         content = request.get_json(force=True, cache=False)
         job_id = content.get('id', '')
         database_api.delete_job(email, job_id)
-        return to_json('', 204)
+        return json_response('', 204)
     else:
         job_id = request.args.get('id', '')
-        job_status = request.args.get('status', '0')
-        return_result = job_status == '0'
         if job_id.startswith('cirro-'):  # precomputed result
             dataset_id = request.args.get('ds_id', '')
             email = get_auth().auth()['email']
             dataset = database_api.get_dataset(email, dataset_id)
-            file_path = get_file_path(os.path.join('uns', job_id + '.json.gz'), dataset['url'])
-            return send_file(file_path)
-        result = database_api.get_job(email=email, job_id=job_id, return_result=return_result)
-        if return_result:
-            result = Response(result, mimetype='application/json')
-            result.headers["Content-Encoding"] = 'gzip'
-        return result
+            dataset['url'] = map_url(dataset['url'])
+            result = dataset_api.get_result(dataset, job_id)
+            return send_file(result)
+        job = database_api.get_job(email=email, job_id=job_id, return_result=True)
+        if isinstance(job, dict) and 'url' in job:
+            url = job['url']
+            content_type = job.get('content-type')
+            if content_type == 'application/h5ad' or content_type == 'application/zarr':
+                import anndata
+                if content_type == 'application/h5ad':
+                    with get_fs(url).open(url, mode='rb') as f:
+                        adata = anndata.read(f)
+                else:
+                    adata = anndata.read_zarr(get_fs(url).get_mapper(url))
+
+                import pandas as pd
+                # output = StringIO()
+                # adata2gct(adata, output)
+                # r = Response(output.getvalue(), mimetype='text/plain')
+                df = pd.DataFrame(adata.X, index=adata.obs.index, columns=adata.var.index)
+                df = df.T.join(adata.var)
+                df.index.name = 'id'
+                return Response(df.reset_index().to_json(double_precision=2, orient='records'),
+                                content_type='application/json')
+            else:
+                # URL to JSON or text
+                return send_file(url)
+        # elif isinstance(job, bytes):
+        #     job = job.decode('ascii')
+        return job
 
 
 @blueprint.route('/jobs', methods=['GET'])
@@ -413,7 +510,7 @@ def handle_jobs():
     email = get_auth().auth()['email']
     database_api = get_database()
     ds_id = request.args.get('id', '')
-    return to_json(database_api.get_jobs(email=email, dataset_id=ds_id))
+    return json_response(database_api.get_jobs(email=email, dataset_id=ds_id))
 
 
 @blueprint.route('/submit_job', methods=['POST'])
@@ -426,4 +523,4 @@ def handle_submit_job():
         job_type = content.get('type')
         job_name = content.get('name')
         return dict(id=submit_job(database_api=database_api, dataset_api=dataset_api, email=email, dataset=dataset,
-            job_name=job_name, job_type=job_type, params=params))
+                                  job_name=job_name, job_type=job_type, params=params))
