@@ -2,21 +2,57 @@ import pandas as pd
 import scipy.sparse
 
 from cirrocumulus.dotplot_aggregator import DotPlotAggregator
-from cirrocumulus.embedding_aggregator import EmbeddingAggregator, get_basis
 from cirrocumulus.feature_aggregator import FeatureAggregator
 from cirrocumulus.ids_aggregator import IdsAggregator
 from cirrocumulus.unique_aggregator import UniqueAggregator
 
 
-def apply_filter(df, data_filter):
-    keep_expr = get_filter_expr(df, data_filter)
-    return df[keep_expr] if keep_expr is not None else df
+def get_mask(dataset_api, dataset, data_filters):
+    measures = set()
+    dimensions = set()
+    basis = set()
+    for data_filter in data_filters:
+        _measures, _dimensions, _basis = data_filter_keys(data_filter)
+        measures.update(_measures)
+        dimensions.update(_dimensions)
+        basis.update(_basis)
+    keys = get_type_to_measures(measures)
+    keys['obs'] = list(dimensions)
+    keys['basis'] = list(basis)
+    adata = dataset_api.read_dataset(keys=keys, dataset=dataset)
+    result = []
+    for data_filter in data_filters:
+        result.append(get_filter_expr(adata, data_filter))
+    return result, adata
 
 
-def get_filter_expr(df, data_filter):
-    if df is None:
-        raise ValueError('df is None')
+def apply_filter(adata, data_filter):
+    keep_expr = get_filter_expr(adata, data_filter)
+    return adata[keep_expr] if keep_expr is not None else adata
 
+
+def get_filter_str(data_filter):
+    user_filters = data_filter.get('filters', [])
+    combine_filters = data_filter.get('combine', 'and')
+    s = []
+    for filter_obj in user_filters:
+        field = filter_obj[0]
+        op = filter_obj[1]
+        value = filter_obj[2]
+        if op == 'in':
+            op = ''
+        if not isinstance(field, dict) and not field == '__index':
+            if isinstance(value, list):
+                value = '_'.join(value)
+                s.append(field + '-' + value)
+            else:
+                return None
+        else:
+            return None
+    return ''.join(s).replace('/', '-').replace(' ', '-').replace('\\', '-')
+
+
+def get_filter_expr(adata, data_filter):
     keep_expr = None
     if data_filter is not None:
         user_filters = data_filter.get('filters', [])
@@ -26,45 +62,42 @@ def get_filter_expr(df, data_filter):
             op = filter_obj[1]
             value = filter_obj[2]
             if isinstance(field, dict):  # selection box
-
-                selected_points_basis = get_basis(field['basis'], field.get('nbins'),
-                    field.get('agg'), field.get('ndim', '2'), field.get('precomputed', False))
+                # selected_points_basis = get_basis(field['basis'], field.get('nbins'),
+                #                                   field.get('agg'), field.get('ndim', '2'),
+                #                                   field.get('precomputed', False))
 
                 if 'points' in value:
                     p = value['points']
-                    field = selected_points_basis['full_name'] if selected_points_basis[
-                                                                      'nbins'] is not None else 'index'
-                    if field == 'index':
-                        keep = df.index.isin(p)
-                    else:
-                        keep = df[field].isin(p)
-                else:
-                    keep = None
-                    for p in value['path']:
-                        if 'z' in p:  # 3d
-                            selection_keep = \
-                                (df[selected_points_basis['coordinate_columns'][0]] >= p['x']) & \
-                                (df[selected_points_basis['coordinate_columns'][0]] <= p['x'] + p['width']) & \
-                                (df[selected_points_basis['coordinate_columns'][1]] >= p['y']) & \
-                                (df[selected_points_basis['coordinate_columns'][1]] <= p['y'] + p[
-                                    'height']) & \
-                                (df[selected_points_basis['coordinate_columns'][2]] >= p['z']) & \
-                                (df[selected_points_basis['coordinate_columns'][2]] <= p['z'] + p['depth'])
-                        else:
-                            selection_keep = \
-                                (df[selected_points_basis['coordinate_columns'][0]] >= p['x']) & \
-                                (df[selected_points_basis['coordinate_columns'][0]] <= p['x'] + p['width']) & \
-                                (df[selected_points_basis['coordinate_columns'][1]] >= p['y']) & \
-                                (df[selected_points_basis['coordinate_columns'][1]] <= p['y'] + p['height'])
+                    # list of indices
+                    keep = pd.RangeIndex(adata.shape[0]).isin(p)
 
-                    keep = selection_keep | keep if keep is not None else selection_keep
+                # else:
+                #     keep = None
+                #     for p in value['path']:
+                #         if 'z' in p:  # 3d
+                #             selection_keep = \
+                #                 (df[selected_points_basis['coordinate_columns'][0]] >= p['x']) & \
+                #                 (df[selected_points_basis['coordinate_columns'][0]] <= p['x'] + p['width']) & \
+                #                 (df[selected_points_basis['coordinate_columns'][1]] >= p['y']) & \
+                #                 (df[selected_points_basis['coordinate_columns'][1]] <= p['y'] + p[
+                #                     'height']) & \
+                #                 (df[selected_points_basis['coordinate_columns'][2]] >= p['z']) & \
+                #                 (df[selected_points_basis['coordinate_columns'][2]] <= p['z'] + p['depth'])
+                #         else:
+                #             selection_keep = \
+                #                 (df[selected_points_basis['coordinate_columns'][0]] >= p['x']) & \
+                #                 (df[selected_points_basis['coordinate_columns'][0]] <= p['x'] + p['width']) & \
+                #                 (df[selected_points_basis['coordinate_columns'][1]] >= p['y']) & \
+                #                 (df[selected_points_basis['coordinate_columns'][1]] <= p['y'] + p['height'])
+
+                # keep = selection_keep | keep if keep is not None else selection_keep
 
             elif field == '__index':
                 import numpy as np
-                keep = np.zeros(len(df), dtype=bool)
+                keep = np.zeros(adata.shape[0], dtype=bool)
                 keep[value] = True
             else:
-                series = df[field]
+                series = adata.obs[field] if field in adata.obs else adata[:, adata.var.index == field].X[:, 0]
                 if op == 'in':
                     keep = (series.isin(value)).values
                 elif op == '>':
@@ -108,7 +141,7 @@ def precomputed_summary(dataset_api, dataset, obs_measures, var_measures, dimens
 def precomputed_grouped_stats(dataset_api, dataset, var_measures, dimensions):
     if (len(var_measures)) > 0 and len(dimensions) > 0:
         return dataset_api.read_precomputed_grouped_stats(dataset,
-            var_keys=var_measures, obs_keys=dimensions)
+                                                          var_keys=var_measures, obs_keys=dimensions)
     return []
 
 
@@ -117,7 +150,7 @@ def precomputed_embedding(dataset_api, dataset, basis, obs_measures, var_measure
     if (len(obs_measures) + len(var_measures) + len(dimensions)) == 0:
         obs_measures = ['__count']
     return dataset_api.read_precomputed_basis(dataset, obs_keys=obs_measures + dimensions, var_keys=var_measures,
-        basis=basis)
+                                              basis=basis)
 
 
 def get_var_name_type(key):
@@ -161,7 +194,7 @@ def handle_export_dataset_filters(dataset_api, dataset, data_filters):
         filter_names.append(data_filter_obj['name'])
         reformatted_filters.append(filter_value)
 
-    df = get_df(dataset_api, dataset, measures=['obs/index'], data_filters=reformatted_filters)
+    df = get_adata(dataset_api, dataset, measures=['obs/index'], data_filters=reformatted_filters)
     result_df = pd.DataFrame(index=df['index'])
     for i in range(len(reformatted_filters)):
         data_filter = reformatted_filters[i]
@@ -178,26 +211,11 @@ def handle_export_dataset_filters(dataset_api, dataset, data_filters):
 def handle_data(dataset_api, dataset, embedding_list=None, values=None, grouped_stats=None, stats=None, selection=None):
     dimensions = set()
     measures = set()
-    basis_list = []
     basis_keys = set()
     if embedding_list is not None:
         for embedding in embedding_list:
-            precomputed = embedding.get('precomputed', False)
-            nbins = check_bin_input(embedding.get('nbins', None))
-            agg_function = embedding.get('agg', 'max')
-            ndim = embedding.get('ndim', '2')
-            coords = embedding.get('coords', True)
-            basis = get_basis(embedding.get('basis'), nbins=nbins, agg=agg_function, dimensions=ndim,
-                precomputed=precomputed, coords=coords)
-            if nbins is not None:
-                dimensions.update(embedding.get('dimensions', []))
-                measures.update(embedding.get('measures', []))
+            basis_keys.add(embedding['name'])
 
-            if basis['full_name'] not in basis_keys:
-                basis_list.append(basis)
-                basis_keys.add(basis['full_name'])
-
-            embedding['basis'] = basis
     if values is not None:
         dimensions.update(values.get('dimensions', []))
         measures.update(values.get('measures', []))
@@ -211,17 +229,9 @@ def handle_data(dataset_api, dataset, embedding_list=None, values=None, grouped_
         dimensions.update(selection.get('dimensions', []))
         measures.update(selection.get('measures', []))
         selection_embeddings = selection.get('embeddings', [])
-        embeddings_mapped = []
-        for embedding in selection_embeddings:
-            basis = get_basis(embedding.get('basis'), nbins=embedding.get('nbins'), agg=embedding.get('agg', 'max'),
-                dimensions=embedding.get('ndim', '2'),
-                precomputed=embedding.get('precomputed', False))
-            embeddings_mapped.append(basis)
-        selection['embeddings'] = embeddings_mapped
-        for basis_obj in selected_points_filter_basis_list + embeddings_mapped:
-            if basis_obj['full_name'] not in basis_keys:
-                basis_list.append(basis_obj)
-                basis_keys.add(basis_obj['full_name'])
+
+        for embedding in selected_points_filter_basis_list + selection_embeddings:
+            basis_keys.add(embedding['name'])
 
     if grouped_stats is not None:
         grouped_stats_dimensions = grouped_stats.get('dimensions', [])
@@ -237,70 +247,58 @@ def handle_data(dataset_api, dataset, embedding_list=None, values=None, grouped_
 
     keys = get_type_to_measures(measures)
     keys['obs'] += list(dimensions)
-    keys['basis'] = basis_list
-    df = dataset_api.read_dataset(dataset=dataset, keys=keys)
-    for basis_obj in basis_list:
-        if not basis_obj['precomputed'] and basis_obj['nbins'] is not None:
-            EmbeddingAggregator.convert_coords_to_bin(df=df,
-                nbins=basis_obj['nbins'],
-                bin_name=basis_obj['full_name'],
-                coordinate_columns=basis_obj['coordinate_columns'])
+    keys['basis'] = list(basis_keys)
+    adata = dataset_api.read_dataset(dataset=dataset, keys=keys)
     results = {}
     if values is not None:
         dimensions = values.get('dimensions', [])
         measures = values.get('measures', [])
         type2measures = get_type_to_measures(measures)
-        measures = []
-        for v in type2measures.values():
-            measures += v
-
-        result = EmbeddingAggregator(
-            measures=measures,
-            dimensions=dimensions,
-            nbins=None,
-            basis=None,
-            coords=False,
-            agg_function=None,
-            quick=True).execute(df)
-        results['values'] = result['values']
-
-    if embedding_list is not None:
-        results['embeddings'] = []
-        for embedding in embedding_list:
-            dimensions = embedding.get('dimensions', [])
-            measures = embedding.get('measures', [])
-            type2measures = get_type_to_measures(measures)
-            basis = embedding['basis']
-            if basis['precomputed']:
-                result = precomputed_embedding(dataset_api, dataset, basis, type2measures['obs'],
-                    type2measures['X'], dimensions)
+        results['values'] = {}
+        for key in type2measures['obs'] + dimensions:
+            series = adata.obs[key]
+            results['values'][key] = series
+            if pd.api.types.is_categorical_dtype(series):
+                results['values'][key] = dict(values=series.values.codes, categories=series.cat.categories.values)
             else:
-                result = EmbeddingAggregator(
-                    measures=type2measures['obs'] + type2measures['X'],
-                    dimensions=dimensions,
-                    nbins=basis['nbins'],
-                    basis=basis,
-                    coords=basis['coords'],
-                    agg_function=basis['agg'],
-                    quick=True).execute(df)
-            results['embeddings'].append(result)
+                results['values'][key] = series
+
+        if adata.uns.get('X_module') is not None:
+            adata_modules = adata.uns['X_module']
+            for i in range(len(adata_modules.var.index)):
+                x = adata_modules.X[:, i]
+                results['values'][adata_modules.var.index[i]] = x
+        if adata.X is not None:
+            is_sparse = scipy.sparse.issparse(adata.X)
+            for i in range(len(adata.var.index)):
+                x = adata.X[:, i]
+                if is_sparse:
+                    indices = x.indices
+                    data = x.data
+                    results['values'][adata.var.index[i]] = dict(indices=indices, values=data)
+                else:
+                    results['values'][adata.var.index[i]] = x
+
+    if embedding_list is not None and len(embedding_list) > 0:
+        results['embeddings'] = []
+        for key in adata.obsm.keys():
+            m = adata.obsm[key]
+            ndim = m.shape[1]
+            coordinates = dict()
+            embedding = dict(name=key, dimensions=ndim, coordinates=coordinates)
+            results['embeddings'].append(embedding)
+            for i in range(ndim):
+                coordinates['{}_{}'.format(key, i + 1)] = m[:, i]
+
     if grouped_stats is not None:
-        if dataset_api.has_precomputed_stats(dataset):
-            results['distribution'] = precomputed_grouped_stats(dataset_api, dataset, grouped_stats.get('measures', []),
-                grouped_stats.get('dimensions', []))
-        else:
-            results['distribution'] = DotPlotAggregator(var_measures=grouped_stats.get('measures', []),
-                dimensions=grouped_stats.get('dimensions', [])).execute(df)
+        results['distribution'] = DotPlotAggregator(var_measures=grouped_stats.get('measures', []),
+                                                    dimensions=grouped_stats.get('dimensions', [])).execute(adata)
     if stats is not None:
         dimensions = stats.get('dimensions', [])
         measures = stats.get('measures', [])
         type2measures = get_type_to_measures(measures)
-        if dataset_api.has_precomputed_stats(dataset):
-            result['summary'] = precomputed_summary(dataset_api, dataset, type2measures['obs'],
-                type2measures['X'], dimensions)
-        else:
-            results['summary'] = FeatureAggregator(type2measures['obs'], type2measures['X'],
-                dimensions).execute(df)
+        results['summary'] = FeatureAggregator(type2measures['obs'], type2measures['X'],
+                                               dimensions).execute(adata)
     if selection is not None:
         results['selection'] = {}
         dimensions = selection.get('dimensions', [])
@@ -308,18 +306,17 @@ def handle_data(dataset_api, dataset, embedding_list=None, values=None, grouped_
         type2measures = get_type_to_measures(measures)
         # basis_list = selection.get('basis', [])
         selection_embeddings = selection.get('embeddings', [])
-        df = apply_filter(df, data_filter)
+        df = apply_filter(adata, data_filter)
         if len(selection_embeddings) > 0:
             results['selection']['coordinates'] = {}
             for embedding in selection_embeddings:
-                results['selection']['coordinates'][embedding['full_name']] = UniqueAggregator(
-                    embedding['full_name'] if embedding['nbins'] is not None else 'index').execute(df)
+                results['selection']['coordinates'][embedding['name']] = UniqueAggregator('index').execute(df)
         var_measures = type2measures['X']
         results['selection']['summary'] = FeatureAggregator(type2measures['obs'], type2measures['X'],
-            dimensions).execute(df)
+                                                            dimensions).execute(df)
         if len(dimensions) > 0 and len(var_measures) > 0:
             results['selection']['distribution'] = DotPlotAggregator(var_measures=var_measures,
-                dimensions=[dimensions]).execute(df)
+                                                                     dimensions=[dimensions]).execute(df)
         results['selection']['count'] = df.shape[0]
 
     return results
@@ -330,7 +327,7 @@ def handle_selection_ids(dataset_api, dataset, data_filter):
     return {'ids': IdsAggregator().execute(df)}
 
 
-def get_df(dataset_api, dataset, embeddings=[], measures=[], dimensions=[], data_filters=[]):
+def get_adata(dataset_api, dataset, embeddings=[], measures=[], dimensions=[], data_filters=[]):
     type2measures = get_type_to_measures(measures)
     var_keys_filter = []
     obs_keys_filter = []
@@ -342,31 +339,25 @@ def get_df(dataset_api, dataset, embeddings=[], measures=[], dimensions=[], data
         obs_keys_filter += _obs_keys_filter
     obs_keys = list(set(type2measures['obs'] + obs_keys_filter + dimensions))
     var_keys = list(set(type2measures['X'] + var_keys_filter))
-    basis_objs = []
     basis_keys = set()
     for embedding in embeddings:
-        basis_obj = get_basis(embedding['basis'], embedding.get('nbins'), embedding.get('agg'),
-            embedding.get('ndim', '2'), embedding.get('precomputed', False))
-        if basis_obj['full_name'] not in basis_keys:
-            basis_objs.append(basis_obj)
-            basis_keys.add(basis_obj['full_name'])
-    for basis_obj in selected_points_filter_basis_list:
-        if basis_obj['full_name'] not in basis_keys:
-            basis_objs.append(basis_obj)
-            basis_keys.add(basis_obj['full_name'])
-    df = dataset_api.read_dataset(dataset=dataset, keys=dict(obs=obs_keys, X=var_keys, basis=basis_objs))
-    for basis_obj in basis_objs:
-        if not basis_obj['precomputed'] and basis_obj['nbins'] is not None:
-            EmbeddingAggregator.convert_coords_to_bin(df=df,
-                nbins=basis_obj['nbins'],
-                bin_name=basis_obj['full_name'],
-                coordinate_columns=basis_obj['coordinate_columns'])
-    return df
+        basis_keys.add(embedding['name'])
+    for embedding in selected_points_filter_basis_list:
+        basis_keys.add(embedding['name'])
+
+    adata = dataset_api.read_dataset(dataset=dataset, keys=dict(obs=obs_keys, X=var_keys, basis=list(basis_keys)))
+    # for basis_obj in basis_objs:
+    #     if not basis_obj['precomputed'] and basis_obj['nbins'] is not None:
+    #         EmbeddingAggregator.convert_coords_to_bin(df=df,
+    #                                                   nbins=basis_obj['nbins'],
+    #                                                   bin_name=basis_obj['full_name'],
+    #                                                   coordinate_columns=basis_obj['coordinate_columns'])
+    return adata
 
 
 def get_selected_data(dataset_api, dataset, embeddings=[], measures=[], dimensions=[], data_filter=None):
-    df = get_df(dataset_api, dataset, embeddings, measures, dimensions,
-        [data_filter] if data_filter is not None else [])
+    df = get_adata(dataset_api, dataset, embeddings, measures, dimensions,
+                   [data_filter] if data_filter is not None else [])
     df = apply_filter(df, data_filter)
     return df
 
@@ -374,7 +365,7 @@ def get_selected_data(dataset_api, dataset, embeddings=[], measures=[], dimensio
 def data_filter_keys(data_filter):
     var_keys = set()
     obs_keys = set()
-    basis_list = []
+    basis_keys = set()
     if data_filter is not None:
         user_filters = data_filter.get('filters', [])
 
@@ -382,9 +373,7 @@ def data_filter_keys(data_filter):
             user_filter = user_filters[i]
             key = user_filter[0]
             if isinstance(key, dict):
-                basis = get_basis(key['basis'], key.get('nbins'), key.get('agg'),
-                    key.get('ndim', '2'), key.get('precomputed', False))
-                basis_list.append(basis)
+                basis_keys.add(key['name'])
             elif key == '__index':
                 continue
             else:
@@ -394,4 +383,4 @@ def data_filter_keys(data_filter):
                     var_keys.add(name)
                 else:
                     obs_keys.add(name)
-    return list(var_keys), list(obs_keys), basis_list
+    return list(var_keys), list(obs_keys), list(basis_keys)
