@@ -1,6 +1,6 @@
 import os
 
-from .file_system_adapter import FileSystemAdapter
+from cirrocumulus.util import get_fs
 
 
 def get_path(dataset, dataset_path):
@@ -16,9 +16,8 @@ def get_path(dataset, dataset_path):
 class DatasetAPI:
     def __init__(self):
         self.suffix_to_provider = {}
-        self.fs_adapter = FileSystemAdapter()
         self.default_provider = None
-        self.cached_schema = None
+        self.cached_dataset_info = None
         self.cached_dataset_id = None
 
     def get_dataset_provider(self, path):
@@ -26,7 +25,6 @@ class DatasetAPI:
         if index == -1:
             return self.default_provider
         suffix = path[index + 1:].lower()
-
         provider = self.suffix_to_provider.get(suffix)
         return provider if provider is not None else self.default_provider
 
@@ -37,45 +35,34 @@ class DatasetAPI:
         for suffix in suffixes:
             self.suffix_to_provider[suffix.lower()] = provider
 
-    def schema(self, dataset):
+    def get_dataset_info(self, dataset):
         dataset_id = dataset['id']
         if self.cached_dataset_id == dataset_id:
-            return self.cached_schema
+            dataset_info = self.cached_dataset_info
+        else:
+            path = dataset['url']
+            provider = self.get_dataset_provider(path)
+            dataset_info = provider.get_dataset_info(get_fs(path), path)
+            self.cached_dataset_info = dataset_info
+            self.cached_dataset_id = dataset_id
+        return dataset_info
+
+    def get_schema(self, dataset):
         path = dataset['url']
         provider = self.get_dataset_provider(path)
-        schema_dict = provider.schema(self.fs_adapter.get_fs(path), path)
+        schema_dict = provider.get_schema(get_fs(path), path)
         if 'summary' in dataset:
             schema_dict['summary'] = dataset['summary']
         if 'markers' in schema_dict:
             schema_dict['markers_read_only'] = schema_dict.pop('markers')
-        self.cached_schema = schema_dict
-        self.cached_dataset_id = dataset['id']
         return schema_dict
 
-    def has_precomputed_stats(self, dataset):
-        path = dataset['url']
-        provider = self.get_dataset_provider(path)
-        return provider.has_precomputed_stats(self.fs_adapter.get_fs(path), path, dataset)
-
-    def read_precomputed_stats(self, dataset, obs_keys=[], var_keys=[]):
-        path = dataset['url']
-        provider = self.get_dataset_provider(path)
-        return provider.read_precomputed_stats(self.fs_adapter.get_fs(path), path, obs_keys=obs_keys, var_keys=var_keys)
-
-    def read_precomputed_grouped_stats(self, dataset, obs_keys=[], var_keys=[]):
-        path = dataset['url']
-        provider = self.get_dataset_provider(path)
-        return provider.read_precomputed_grouped_stats(self.fs_adapter.get_fs(path), path, obs_keys=obs_keys,
-            var_keys=var_keys)
-
-    def read_precomputed_basis(self, dataset, obs_keys=[], var_keys=[], basis=None):
-        path = dataset['url']
-        provider = self.get_dataset_provider(path)
-        return provider.read_precomputed_basis(self.fs_adapter.get_fs(path), path, obs_keys=obs_keys, var_keys=var_keys,
-            basis=basis)
-
     def read_dataset(self, dataset, keys=[]):
         path = dataset['url']
         provider = self.get_dataset_provider(path)
-        return provider.read_dataset(self.fs_adapter.get_fs(path), path, keys=keys, dataset=dataset,
-            schema=self.schema(dataset))
+        return provider.read_dataset(get_fs(path), path, keys=keys, dataset=dataset)
+
+    def get_result(self, dataset, result_id):
+        path = dataset['url']
+        provider = self.get_dataset_provider(path)
+        return provider.get_result(get_fs(path), path, dataset=dataset, result_id=result_id)
