@@ -1,28 +1,38 @@
 import os
 
 from cirrocumulus.anndata_dataset import AnndataDataset
+from cirrocumulus.envir import CIRRO_DATABASE, CIRRO_AUTH, CIRRO_JOB_TYPE, CIRRO_JOB_RESULTS, CIRRO_CELL_ONTOLOGY
 from cirrocumulus.io_util import get_markers, filter_markers, add_spatial, SPATIAL_HELP
-from cirrocumulus.parquet_dataset import ParquetDataset
+from cirrocumulus.local_db_api import LocalDbAPI
+from cirrocumulus.util import get_fs
 
 
-def configure_app(app, list_of_dataset_paths, spatial_directories, backed, marker_paths):
+def configure_app(app, list_of_dataset_paths, spatial_directories, marker_paths):
     from cirrocumulus.api import dataset_api
-    from cirrocumulus.local_db_api import LocalDbAPI
     from cirrocumulus.no_auth import NoAuth
-
-    app.config['AUTH'] = NoAuth()
-    dataset_api.add(ParquetDataset())
-    anndata_dataset = AnndataDataset('r' if backed else None)
+    try:
+        from cirrocumulus.parquet_dataset import ParquetDataset
+        dataset_api.add(ParquetDataset())
+    except ModuleNotFoundError:
+        pass
+    try:
+        from cirrocumulus.zarr_dataset import ZarrDataset
+        dataset_api.add(ZarrDataset())
+    except ModuleNotFoundError:
+        pass
+    app.config[CIRRO_AUTH] = NoAuth()
+    os.environ[CIRRO_JOB_TYPE + 'de'] = 'cirrocumulus.job_api.run_de'
+    anndata_dataset = AnndataDataset()
     dataset_ids = []
     for dataset_paths in list_of_dataset_paths:
         dataset_paths = dataset_paths.split(',')
-        dataset_id = os.path.normpath(dataset_paths[0])
+        dataset_id = dataset_paths[0]
         dataset_ids.append(dataset_id)
         if len(dataset_paths) > 1:
             to_concat = []
             all_ids = None
             for path in dataset_paths:
-                d = anndata_dataset.get_data(path)
+                d = anndata_dataset.get_data(get_fs(path), path)
                 all_ids = d.obs.index.union(all_ids) if all_ids is not None else d.obs.index
                 to_concat.append(d)
             for i in range(len(to_concat)):
@@ -66,20 +76,20 @@ def configure_app(app, list_of_dataset_paths, spatial_directories, backed, marke
             anndata_dataset.add_data(dataset_id, adata)
         dataset_api.add(anndata_dataset)
 
-    app.config['DATABASE'] = LocalDbAPI(dataset_ids)
+    app.config[CIRRO_DATABASE] = LocalDbAPI(dataset_ids)
 
     if spatial_directories is not None and len(spatial_directories) > 0:
         for i in range(len(spatial_directories)):
             spatial_directory = spatial_directories[i]
             if spatial_directory != '':
-                adata = anndata_dataset.get_data(dataset_ids[i])
+                adata = anndata_dataset.get_data(get_fs(dataset_ids[i]), dataset_ids[i])
                 if not add_spatial(adata, spatial_directory):
                     print('No spatial data found in {}'.format(spatial_directory))
 
     if marker_paths is not None and len(marker_paths) > 0:
         markers = get_markers(marker_paths)
         for dataset_id in dataset_ids:
-            d = anndata_dataset.get_data(dataset_id)
+            d = anndata_dataset.get_data(get_fs(dataset_id), dataset_id)
             existing_markers = d.uns.get('markers', [])
             markers += existing_markers
             # remove genes in dict that are not in dataset
@@ -105,33 +115,47 @@ def create_app():
 
 def main(argsv):
     import argparse
+
     parser = argparse.ArgumentParser(description='Run cirrocumulus')
     parser.add_argument('dataset',
-        help='Path to dataset in h5ad, loom, or STAR-Fusion format. Separate multiple datasets with '
-             'a comma instead of a space in order to join datasets by cell id', nargs='+')
+                        help='Path to dataset in h5ad, loom, Seurat, TileDB, zarr, or STAR-Fusion format. Separate multiple datasets with '
+                             'a comma instead of a space in order to join datasets by cell id', nargs='+')
     parser.add_argument('--spatial', help=SPATIAL_HELP, nargs='*')
     parser.add_argument('--markers',
-        help='Path to JSON file that maps name to features. For example {"a":["gene1", "gene2"], "b":["gene3"]}',
-        nargs='*')
-    parser.add_argument('--backed', help='Load h5ad file in backed mode', action='store_true')
+                        help='Path to JSON file that maps name to features. For example {"a":["gene1", "gene2"], "b":["gene3"]}',
+                        nargs='*')
     parser.add_argument('--host',
-        help='Host IP address')  # set to 0.0.0.0 to make it accessible from other computers WITHOUT SECURITY.
+                        help='Host IP address')  # set to 0.0.0.0 to make it accessible from other computers WITHOUT SECURITY.
 
     parser.add_argument('--port', help='Server port', default=5000, type=int)
     parser.add_argument('--no-open', dest='no_open', help='Do not open your web browser', action='store_true')
-
+    parser.add_argument('--results', help='URL to save user computed results (e.g. differential expression)')
+    parser.add_argument('--ontology', help='Path to ontology in OBO format for annotation')
     args = parser.parse_args(argsv)
+    if args.results is not None:
+        os.environ[CIRRO_JOB_RESULTS] = args.results
+    else:
+        os.environ[CIRRO_JOB_RESULTS] = os.path.join(os.path.dirname(args.dataset[0].rstrip('/')), 'results')
+    get_fs(os.environ[CIRRO_JOB_RESULTS]).makedirs(os.environ[CIRRO_JOB_RESULTS], exist_ok=True)
+    if args.ontology is not None:
+        os.environ[CIRRO_CELL_ONTOLOGY] = args.ontology
     app = create_app()
-    # from flask_cors import CORS
-    # CORS(app)
-    configure_app(app, args.dataset, args.spatial, args.backed, args.markers)
+    configure_app(app, args.dataset, args.spatial, args.markers)
     if not args.no_open:
-        import webbrowser
+        import webbrowser, requests
         host = args.host if args.host is not None else 'http://127.0.0.1'
         url = host + ':' + str(args.port)
+        try:
+            if requests.get(url).ok:
+                import sys
+                sys.exit('Address already in use')
+        except:
+            pass
         webbrowser.open(url)
     app.run(host=args.host, port=args.port, debug=False)
 
 
 if __name__ == "__main__":
-    main()
+    import sys
+
+    main(sys.argv[1:])
