@@ -1,24 +1,23 @@
-import {Divider, IconButton, Menu, Tooltip} from '@material-ui/core';
-import AppBar from '@material-ui/core/AppBar';
-import Button from '@material-ui/core/Button';
-import Link from '@material-ui/core/Link';
-import MenuItem from '@material-ui/core/MenuItem';
-import Popover from '@material-ui/core/Popover';
-import {withStyles} from '@material-ui/core/styles';
-import Tab from '@material-ui/core/Tab';
-import Tabs from '@material-ui/core/Tabs';
-import Toolbar from '@material-ui/core/Toolbar';
-import Typography from '@material-ui/core/Typography';
-import AccountCircle from '@material-ui/icons/AccountCircle';
-import Brightness2Icon from '@material-ui/icons/Brightness3';
-import HelpIcon from '@material-ui/icons/Help';
-import MoreVertIcon from '@material-ui/icons/MoreVert';
+import {Divider, IconButton, Menu, Tooltip} from '@mui/material';
+import AppBar from '@mui/material/AppBar';
+import Button from '@mui/material/Button';
+import Link from '@mui/material/Link';
+import MenuItem from '@mui/material/MenuItem';
+import Popover from '@mui/material/Popover';
+import Tab from '@mui/material/Tab';
+import Toolbar from '@mui/material/Toolbar';
+import Typography from '@mui/material/Typography';
+import AccountCircle from '@mui/icons-material/AccountCircle';
+import Brightness2Icon from '@mui/icons-material/Brightness3';
+import HelpIcon from '@mui/icons-material/Help';
+import MoreVertIcon from '@mui/icons-material/MoreVert';
 import ReactMarkdown from 'markdown-to-jsx';
 import React from 'react';
 import {connect} from 'react-redux';
 import {
     DELETE_DATASET_DIALOG,
     EDIT_DATASET_DIALOG,
+    getDatasetStateJson,
     HELP_DIALOG,
     IMPORT_DATASET_DIALOG,
     login,
@@ -28,55 +27,24 @@ import {
     setDialog,
     setMessage,
     setSavedDatasetState,
-    setTab,
+    setTab
 } from './actions';
 import {drawerWidth} from './App';
 import CirroIcon from './CirroIcon';
 import DatasetSelector from './DatasetSelector';
 import {intFormat} from './formatters';
 import {
-    DEFAULT_DARK_MODE,
-    DEFAULT_LABEL_FONT_SIZE,
-    DEFAULT_LABEL_STROKE_WIDTH,
-    DEFAULT_MARKER_OPACITY,
-    DEFAULT_SHOW_FOG,
-    DEFAULT_UNSELECTED_MARKER_OPACITY
-} from "./reducers";
-import {copyToClipboard, FEATURE_TYPE, REACT_MD_OVERRIDES} from './util';
+    copyToClipboard,
+    FEATURE_TYPE,
+    REACT_MD_OVERRIDES,
+    SERVER_CAPABILITY_ADD_DATASET,
+    SERVER_CAPABILITY_DELETE_DATASET,
+    SERVER_CAPABILITY_EDIT_DATASET
+} from './util';
+import Box from '@mui/material/Box';
+import Tabs from '@mui/material/Tabs';
 
 
-const styles = theme => ({
-    root: {
-        display: 'flex',
-        flexWrap: 'wrap',
-        'flex-direction': 'column',
-    },
-    appBar: {
-        width: `calc(100% - ${drawerWidth}px)`,
-        marginLeft: drawerWidth,
-    },
-});
-const AntTab = withStyles(theme => ({
-    root: {
-        minWidth: 50,
-        textTransform: 'none',
-        fontWeight: theme.typography.fontWeightRegular,
-        marginRight: theme.spacing(0),
-        '&:hover': {
-            color: '#40a9ff',
-            opacity: 1,
-        },
-        '&$selected': {
-            color: '#1890ff',
-            fontWeight: theme.typography.fontWeightMedium,
-        },
-        '&:focus': {
-            color: '#40a9ff',
-        },
-    },
-    selected: {},
-}))(props => <Tab disableRipple {...props} />);
-
 class AppHeader extends React.PureComponent {
 
     constructor(props) {
@@ -85,7 +53,7 @@ class AppHeader extends React.PureComponent {
             userMenuOpen: false,
             userMenuAnchorEl: null,
             moreMenuOpen: false,
-            moreMenuAnchorEl: null,
+            moreMenuAnchorEl: null
         };
 
     }
@@ -138,113 +106,7 @@ class AppHeader extends React.PureComponent {
     };
 
     getLinkJson = () => {
-        const {
-            chartOptions,
-            combineDatasetFilters,
-            activeFeature,
-            dataset,
-            embeddingLabels,
-            distributionPlotOptions,
-            distributionPlotInterpolator,
-            embeddings,
-            searchTokens,
-            datasetFilter,
-            interpolator,
-            markerOpacity,
-            pointSize,
-            unselectedMarkerOpacity,
-            distributionData
-        } = this.props;
-
-        let json = {
-            dataset: dataset.id,
-            embeddings: embeddings.map(embedding => {
-                if (embedding.bin) {
-                    embedding = Object.assign({}, embedding);
-                    delete embedding._bin;
-                    return embedding;
-                } else {
-                    return {name: embedding.name, dimensions: embedding.dimensions};
-                }
-
-            })
-        };
-
-        const chartRef = chartOptions.ref;
-        if (json.embeddings.length > 0 && chartRef != null && chartRef.scatterPlot) {
-            json.camera = chartRef.scatterPlot.getCameraDef();
-
-        }
-        if (activeFeature != null) {
-            json.activeFeature = activeFeature;
-        }
-        let jsonChartOptions = {};
-
-        const defaultChartOptions = {
-            showFog: DEFAULT_SHOW_FOG, darkMode: DEFAULT_DARK_MODE,
-            labelFontSize: DEFAULT_LABEL_FONT_SIZE,
-            labelStrokeWidth: DEFAULT_LABEL_STROKE_WIDTH
-        };
-
-        for (let key in defaultChartOptions) {
-            let value = chartOptions[key];
-            if (value !== defaultChartOptions[key]) {
-                jsonChartOptions[key] = value;
-            }
-        }
-        if (pointSize !== 1) {
-            json.pointSize = pointSize;
-        }
-
-        json.chartOptions = jsonChartOptions;
-
-
-        if (searchTokens.length > 0) {
-            json.q = searchTokens;
-        }
-
-        let datasetFilterJson = {};
-        for (let key in datasetFilter) {
-            let filterObject = datasetFilter[key];
-            if (Array.isArray(filterObject)) { // brush filter
-                const array = [];
-                filterObject.forEach(brush => {
-                    const brushJson = Object.assign({}, brush);
-                    brushJson.indices = Array.from(brush.indices);
-                    array.push(brushJson);
-                });
-                datasetFilterJson[key] = array;
-            } else if (filterObject.operation === 'in') {
-                datasetFilterJson[key] = {operation: filterObject.operation, value: filterObject.value};
-            } else {
-                if (filterObject.operation !== '' && !isNaN(filterObject.value) && filterObject.value != null) {
-                    datasetFilterJson[key] = {operation: filterObject.operation, value: filterObject.value};
-                }
-            }
-        }
-        if (Object.keys(datasetFilterJson).length > 0) {
-            json.datasetFilter = datasetFilterJson;
-        }
-        json.combineDatasetFilters = combineDatasetFilters;
-        if (markerOpacity !== DEFAULT_MARKER_OPACITY) {
-            json.markerOpacity = markerOpacity;
-        }
-        if (unselectedMarkerOpacity !== DEFAULT_UNSELECTED_MARKER_OPACITY) {
-            json.unselectedMarkerOpacity = unselectedMarkerOpacity;
-        }
-
-        if (distributionData && distributionData.length > 0) {
-            json.distributionPlotOptions = distributionPlotOptions;
-            json.distributionPlotInterpolator = Object.assign({}, distributionPlotInterpolator, {value: null});
-        }
-
-        const interpolatorJson = Object.assign({}, interpolator, {value: null});
-        json.colorScheme = interpolatorJson;
-
-        if (embeddingLabels.length > 0) {
-            json.embeddingLabels = embeddingLabels;
-        }
-        return json;
+        return getDatasetStateJson(this.props);
     };
 
     handleDataset = (id) => {
@@ -308,7 +170,6 @@ class AppHeader extends React.PureComponent {
             jobResults,
             email,
             selection,
-            classes,
             searchTokens,
             serverInfo,
             tab,
@@ -320,12 +181,14 @@ class AppHeader extends React.PureComponent {
         const shape = dataset != null && dataset.shape != null ? dataset.shape : null;
         const hasSelection = dataset != null && shape != null && shape[0] > 0 && selection.size > 0;
         const obsCat = searchTokens.filter(item => item.type === FEATURE_TYPE.OBS_CAT).map(item => item.value);
-        const showNewDataset = user != null && user.importer && !loadingApp.loading;
-        const showEditDeleteDataset = dataset !== null && dataset.owner && !loadingApp.loading;
-        const showMoreMenu = (showNewDataset || dataset != null) && !loadingApp.loading;
+        const showAddDataset = user != null && user.importer && !loadingApp.loading && serverInfo.capabilities.has(SERVER_CAPABILITY_ADD_DATASET);
+        const showEditDataset = dataset !== null && dataset.owner && !loadingApp.loading && serverInfo.capabilities.has(SERVER_CAPABILITY_EDIT_DATASET);
+        const showDeleteDataset = dataset !== null && dataset.owner && !loadingApp.loading && serverInfo.capabilities.has(SERVER_CAPABILITY_DELETE_DATASET);
+
+        const showMoreMenu = (showAddDataset || showEditDataset || showDeleteDataset || dataset != null) && !loadingApp.loading;
         const isSignedOut = !loadingApp.loading && email == null && serverInfo.clientId !== '';
         return (
-            <AppBar position="fixed" color="default" className={classes.appBar}>
+            <AppBar position="fixed" sx={{width: `calc(100% - ${drawerWidth}px)`, ml: `${drawerWidth}px`}}>
                 <Toolbar variant="dense" style={{paddingLeft: 6}}>
                     {dataset != null && <Popover
                         id={"dataset-details"}
@@ -334,33 +197,33 @@ class AppHeader extends React.PureComponent {
                         onClose={this.handleCloseDatasetDetails}
                         anchorOrigin={{
                             vertical: 'bottom',
-                            horizontal: 'center',
+                            horizontal: 'center'
                         }}
                         transformOrigin={{
                             vertical: 'top',
-                            horizontal: 'center',
+                            horizontal: 'center'
                         }}
                     >
 
-                        <div style={{width: 500, padding: '1em'}}>
-                            <Typography className={classes.typography}>
+                        <Box style={{width: 500, padding: '1em'}}>
+                            <Typography>
                                 {dataset.name}
                             </Typography>
                             <Divider/>
-                            {dataset.species && <Typography className={classes.typography}>
+                            {dataset.species && <Typography>
                                 Species: {dataset.species}
                             </Typography>}
 
-                            {dataset.title && <Typography className={classes.typography}>
+                            {dataset.title && <Typography>
                                 Title: {dataset.title}
                             </Typography>}
                             {dataset.description &&
                             <>Description: <ReactMarkdown options={{overrides: REACT_MD_OVERRIDES}}
                                                           children={dataset.description}/></>}
-                            {!process.env.REACT_APP_STATIC === 'true' && <Typography className={classes.typography}>
+                            {!process.env.REACT_APP_STATIC === 'true' && <Typography>
                                 URL: {dataset.url}
                             </Typography>}
-                        </div>
+                        </Box>
                     </Popover>
                     }
                     {dataset == null && <Typography variant="h5">
@@ -369,6 +232,7 @@ class AppHeader extends React.PureComponent {
                     {dataset && <CirroIcon/>}
                     {dataset &&
                     <Link
+                        color="inherit"
                         style={{
                             paddingLeft: 6,
                             whiteSpace: 'nowrap',
@@ -390,18 +254,18 @@ class AppHeader extends React.PureComponent {
                         {shape != null && intFormat(shape[0]) + ' cells'}
                     </small>}
 
-                    {dataset != null && <Tabs
-                        value={tab}
-                        indicatorColor="primary"
-                        textColor="primary"
-                        onChange={this.handleTabChange}
-                    >
-                        <AntTab value="embedding" label="Embeddings"/>
-                        <AntTab value="distribution" label="Distributions" disabled={distributionData.length === 0}/>
-                        <AntTab value="composition" label="Composition" disabled={obsCat.length < 2}/>
-                        {jobResults.length > 0 && <AntTab value="results" label="Results"/>}
-                    </Tabs>}
 
+                    {dataset != null &&
+                    <Tabs textColor="inherit"
+                          indicatorColor="secondary"
+                          value={tab} onChange={this.handleTabChange}>
+                        <Tab data-testid="embedding-tab" value="embedding" label="Embeddings"/>
+                        <Tab data-testid="distributions-tab" value="distribution" label="Distributions"
+                             disabled={distributionData.length === 0}/>
+                        <Tab data-testid="composition-tab" value="composition" label="Composition"
+                             disabled={obsCat.length < 2}/>
+                        {jobResults.length > 0 && <Tab data-testid="results-tab" value="results" label="Results"/>}
+                    </Tabs>}
 
                     <div style={{marginLeft: 'auto', whiteSpace: 'nowrap', overflow: 'hidden'}}>
                         {/*{serverInfo.brand && <Typography variant="h5"*/}
@@ -425,8 +289,11 @@ class AppHeader extends React.PureComponent {
 
                         {!loadingApp.loading && !isSignedOut && <DatasetSelector onChange={this.handleDataset}/>}
                         {showMoreMenu && <Tooltip title={'More'}>
-                            <IconButton aria-label="Menu" aria-haspopup="true"
-                                        onClick={this.handleMoreMenuOpen}>
+                            <IconButton
+                                aria-label="Menu"
+                                aria-haspopup="true"
+                                onClick={this.handleMoreMenuOpen}
+                                size="large">
                                 <MoreVertIcon/>
                             </IconButton>
                         </Tooltip>}
@@ -434,40 +301,46 @@ class AppHeader extends React.PureComponent {
                                                anchorEl={this.state.moreMenuAnchorEl}
                                                anchorOrigin={{
                                                    vertical: 'top',
-                                                   horizontal: 'right',
+                                                   horizontal: 'right'
                                                }}
 
                                                transformOrigin={{
                                                    vertical: 'top',
-                                                   horizontal: 'right',
+                                                   horizontal: 'right'
                                                }} open={this.state.moreMenuOpen}
                                                onClose={this.handleMoreMenuClose}>
-                            {showNewDataset && <MenuItem onClick={this.handleImportDataset}>
+                            {showAddDataset && <MenuItem onClick={this.handleImportDataset}>
                                 New Dataset
                             </MenuItem>}
 
-                            {showEditDeleteDataset && <MenuItem onClick={this.handleSettings}>Edit Dataset</MenuItem>}
-                            {showEditDeleteDataset && <MenuItem onClick={this.handleDelete}>Delete Dataset</MenuItem>}
-                            {(showNewDataset || showEditDeleteDataset) && dataset != null && <Divider/>}
+                            {showEditDataset && <MenuItem onClick={this.handleSettings}>Edit Dataset</MenuItem>}
+                            {showDeleteDataset && <MenuItem onClick={this.handleDelete}>Delete Dataset</MenuItem>}
+                            {(showAddDataset || showEditDataset || showDeleteDataset) && dataset != null && <Divider/>}
                             {dataset != null && <MenuItem onClick={this.copyLink}>Copy Link </MenuItem>}
                         </Menu>}
 
                         {<Tooltip title={"Toggle Light/Dark Theme"}>
-                            <IconButton edge={false} className={this.props.chartOptions.darkMode ? 'cirro-active' : ''}
-                                        aria-label="Toggle Theme" onClick={() => this.onDarkMode()}>
+                            <IconButton
+                                edge={false}
+                                className={this.props.chartOptions.darkMode ? 'cirro-active' : ''}
+                                aria-label="Toggle Theme"
+                                onClick={() => this.onDarkMode()}
+                                size="large">
                                 <Brightness2Icon/>
                             </IconButton>
                         </Tooltip>}
                         {dataset != null && <Tooltip title={'Help'}>
-                            <IconButton aria-label="Help"
-                                        onClick={this.handleHelp}>
+                            <IconButton aria-label="Help" onClick={this.handleHelp} size="large">
                                 <HelpIcon/>
                             </IconButton>
                         </Tooltip>}
                         {email != null && email !== '' &&
                         <Tooltip title={email}>
-                            <IconButton aria-label="Menu" aria-haspopup="true"
-                                        onClick={this.handleUserMenuOpen}>
+                            <IconButton
+                                aria-label="Menu"
+                                aria-haspopup="true"
+                                onClick={this.handleUserMenuOpen}
+                                size="large">
                                 <AccountCircle/>
                             </IconButton>
                         </Tooltip>}
@@ -476,22 +349,21 @@ class AppHeader extends React.PureComponent {
                               anchorEl={this.state.userMenuAnchorEl}
                               anchorOrigin={{
                                   vertical: 'top',
-                                  horizontal: 'right',
+                                  horizontal: 'right'
                               }}
 
                               transformOrigin={{
                                   vertical: 'top',
-                                  horizontal: 'right',
+                                  horizontal: 'right'
                               }} open={this.state.userMenuOpen}
                               onClose={this.handleUserMenuClose}>
                             <MenuItem onClick={this.handleLogout}>Sign Out</MenuItem>
                         </Menu>}
-                        {isSignedOut && <Button style={{whiteSpace: 'nowrap'}} variant="outlined" color="primary"
+                        {isSignedOut && <Button style={{whiteSpace: 'nowrap'}}  color="inherit"
                                                 onClick={this.props.handleLogin}>Sign In</Button>}
                     </div>
                 </Toolbar>
             </AppBar>
-
         );
     }
 }
@@ -559,8 +431,8 @@ const mapDispatchToProps = (dispatch, ownProps) => {
     };
 };
 
-export default withStyles(styles)(connect(
-    mapStateToProps, mapDispatchToProps,
+export default (connect(
+    mapStateToProps, mapDispatchToProps
 )(AppHeader));
 
 
