@@ -1,17 +1,20 @@
 import {scaleLinear} from 'd3-scale';
 import React from 'react';
+import {CANVAS_FONT} from './ChartUtil';
 import {numberFormat} from './formatters';
 
-export function drawColorScheme(context, width, height, colorScale, label, nsteps = 10, textColor = 'black') {
+export function drawColorScheme(context, colorScale, textColor = 'black', label = true, width = 150, height = 12) {
     let domain = colorScale.domain();
     if (domain[0] === domain[1]) {
         return;
     }
+
+    const nsteps = 10;
+    let gradient = context.createLinearGradient(0, 0, width, height);
+    let valueToFraction = scaleLinear().range([0, 1]).domain(domain).clamp(true);
+
     let value = domain[0];
     let stepSize = (domain[1] - domain[0]) / nsteps;
-    let legendHeight = 18;
-    let gradient = context.createLinearGradient(0, 0, width, legendHeight);
-    let valueToFraction = scaleLinear().range([0, 1]).domain(domain).clamp(true);
     for (let i = 0; i < nsteps; i++) {
         if (i === (nsteps - 1)) {
             value = domain[1];
@@ -24,18 +27,18 @@ export function drawColorScheme(context, width, height, colorScale, label, nstep
         value += stepSize;
     }
     context.fillStyle = gradient;
-    context.fillRect(0, 0, width, legendHeight);
+    context.fillRect(0, 0, width, height);
     context.strokeStyle = 'LightGrey';
-    context.strokeRect(0, 0, width, legendHeight);
-    if (label > 0) {
+    context.strokeRect(0, 0, width, height);
+    if (label) {
 
         context.textBaseline = 'top';
         context.fillStyle = textColor;
 
         context.textAlign = 'left';
-        context.fillText(numberFormat(domain[0]), 0, legendHeight + 2);
+        context.fillText(numberFormat(domain[0]), 0, height + 2);
         context.textAlign = 'right';
-        context.fillText(numberFormat(domain[1]), width, legendHeight + 2);
+        context.fillText(numberFormat(domain[1]), width, height + 2);
     }
 }
 
@@ -63,8 +66,8 @@ class ColorSchemeLegend extends React.PureComponent {
             return;
         }
 
-        context.font = '12px Roboto Condensed,Helvetica,Arial,sans-serif';
-        drawColorScheme(context, width, height, colorScale, this.props.label, 10, textColor);
+        context.font = CANVAS_FONT;
+        drawColorScheme(context, colorScale, textColor, this.props.label, width, height);
         context.setTransform(1, 0, 0, 1, 0, 0);
     }
 
