@@ -3,7 +3,7 @@ import {schemeCategory10} from 'd3-scale-chromatic';
 import React from 'react';
 import {connect} from 'react-redux';
 import CompositionPlot from './CompositionPlot';
-import {FEATURE_TYPE, NATSORT} from './util';
+import {FEATURE_TYPE, getCategoryValue, NATSORT} from './util';
 
 function getColorScale(embeddingData, dimension) {
     let categoryColorScale = null;
@@ -26,10 +26,7 @@ function createSorter(name, categoryOrder, categoricalNames) {
         const nameMap = categoricalNames[name] || {};
         for (let i = 0; i < orderedCategories.length; i++) {
             let value = orderedCategories[i];
-            let newValue = nameMap[value];
-            if (newValue !== undefined) {
-                value = newValue;
-            }
+            value = getCategoryValue(nameMap, value);
             categoryToIndex.set(value, i);
         }
         return (a, b) => categoryToIndex.get(a) - categoryToIndex.get(b);
@@ -40,8 +37,9 @@ function createSorter(name, categoryOrder, categoricalNames) {
 
 function getComposition(dataset, obsCat, cachedData, categoricalNames, selection) {
     const ncategories = obsCat.length;
-
+    const maxCategories = 100;
     if (ncategories >= 2) {
+        let nseries = 0;
         let categoryValues = [];
         let nObs = dataset.shape[0];
         const renamedDimensions = [];
@@ -65,10 +63,7 @@ function getComposition(dataset, obsCat, cachedData, categoricalNames, selection
             for (let categoryIndex = 0; categoryIndex < ncategories - 1; categoryIndex++) {
                 let value = categoryValues[categoryIndex][i];
                 const nameMap = renamedDimensions[categoryIndex];
-                let newValue = nameMap[value];
-                if (newValue !== undefined) {
-                    value = newValue;
-                }
+                value = getCategoryValue(nameMap, value);
                 seriesArray.push(value);
             }
             const seriesKey = seriesArray.join(',');
@@ -77,6 +72,10 @@ function getComposition(dataset, obsCat, cachedData, categoricalNames, selection
                 valueToCounts = {};
                 seriesToSeriesArray[seriesKey] = seriesArray;
                 seriesToValueToCounts[seriesKey] = valueToCounts;
+                nseries++;
+                if (nseries > maxCategories) {
+                    return null;
+                }
             }
             let category = categoryValues[dimensionIndex][i];
             const nameMap = renamedDimensions[dimensionIndex];
@@ -105,6 +104,7 @@ function getComposition(dataset, obsCat, cachedData, categoricalNames, selection
                     return r;
                 }
             }
+            return 0;
         });
 
         let uniqueValuesSet = new Set();
@@ -130,18 +130,16 @@ function CompositionPlots(props) {
         const dimension = obsCat[obsCat.length - 1];
         const colorScale = getColorScale(embeddingData, dimension);
         const composition = getComposition(dataset, obsCat, cachedData, categoricalNames);
-        if (composition == null) {
-            return null;
-        }
+
         const textColor = chartOptions.darkMode ? 'white' : 'black';
         const selectedComposition = selection.size > 0 ? getComposition(dataset, obsCat, cachedData, categoricalNames, selection) : null;
         const title = dimension + ' composition in ' + obsCat.slice(0, obsCat.length - 1).join(', ');
-        return <><CompositionPlot seriesToValueToCounts={composition.seriesToValueToCounts}
-                                  dimension={dimension}
-                                  title={title}
-                                  colorScale={colorScale} series={composition.series}
-                                  uniqueValues={composition.uniqueValues}
-                                  textColor={textColor}/>
+        return <>{composition && <CompositionPlot seriesToValueToCounts={composition.seriesToValueToCounts}
+                                                  dimension={dimension}
+                                                  title={title}
+                                                  colorScale={colorScale} series={composition.series}
+                                                  uniqueValues={composition.uniqueValues}
+                                                  textColor={textColor}/>}
 
             {selectedComposition &&
             <CompositionPlot seriesToValueToCounts={selectedComposition.seriesToValueToCounts}
@@ -150,7 +148,8 @@ function CompositionPlots(props) {
                              subtitle="selection"
                              colorScale={colorScale} series={selectedComposition.series}
                              uniqueValues={selectedComposition.uniqueValues}
-                             textColor={textColor}/>}</>;
+                             textColor={textColor}/>}
+        </>;
     }
     return null;
 }
@@ -171,6 +170,6 @@ const mapDispatchToProps = dispatch => {
 };
 
 export default (connect(
-    mapStateToProps, mapDispatchToProps,
+    mapStateToProps, mapDispatchToProps
 )(CompositionPlots));
 
