@@ -1,9 +1,9 @@
-import Button from '@material-ui/core/Button';
-import Dialog from '@material-ui/core/Dialog';
-import DialogActions from '@material-ui/core/DialogActions';
-import DialogContent from '@material-ui/core/DialogContent';
+import Button from '@mui/material/Button';
+import Dialog from '@mui/material/Dialog';
+import DialogActions from '@mui/material/DialogActions';
+import DialogContent from '@mui/material/DialogContent';
 
-import DialogTitle from '@material-ui/core/DialogTitle';
+import DialogTitle from '@mui/material/DialogTitle';
 import React from 'react';
 import {connect} from 'react-redux';
 import {deleteDataset, setDialog} from './actions';
@@ -40,10 +40,11 @@ class DeleteDatasetDialog extends React.PureComponent {
                     {this.props.dataset && <h3>Are you sure you want to delete {this.props.dataset.name}?</h3>}
                 </DialogContent>
                 <DialogActions>
-                    <Button disabled={this.state.loading} onClick={this.handleClose} color="primary">
+                    <Button disabled={this.state.loading} onClick={this.handleClose}>
                         Cancel
                     </Button>
-                    <Button disabled={this.state.loading} onClick={this.handleDelete} color="primary">
+                    <Button disabled={this.state.loading} onClick={this.handleDelete} variant="contained"
+                            color="primary">
                         Delete
                     </Button>
                 </DialogActions>
