@@ -1,8 +1,7 @@
-import {InputLabel, MenuItem, Select} from '@material-ui/core';
-import Box from '@material-ui/core/Box';
-import FormControl from '@material-ui/core/FormControl';
-import Input from '@material-ui/core/Input';
-import withStyles from '@material-ui/core/styles/withStyles';
+import {InputLabel, MenuItem, Select, Switch} from '@mui/material';
+import Box from '@mui/material/Box';
+import FormControl from '@mui/material/FormControl';
+import withStyles from '@mui/styles/withStyles';
 import {scaleLinear} from 'd3-scale';
 import React from 'react';
 import DotPlotCanvas from './DotPlotCanvas';
@@ -13,6 +12,7 @@ import {mannWhitney} from './MannWhitneyUTest';
 import {
     createColorScale,
     fdr,
+    getCategoryValue,
     INTERPOLATOR_SCALING_MIN_MAX_CATEGORY,
     INTERPOLATOR_SCALING_MIN_MAX_FEATURE,
     INTERPOLATOR_SCALING_NONE,
@@ -21,11 +21,13 @@ import {
 import {Vector} from './Vector';
 import {stats} from './VectorUtil';
 import ViolinPlot from './ViolinPlot';
+import FormHelperText from '@mui/material/FormHelperText';
+import FormControlLabel from '@mui/material/FormControlLabel';
 
 const styles = theme => ({
     formControl: {
         display: 'block',
-        margin: theme.spacing(1),
+        margin: theme.spacing(1)
     }
 });
 
@@ -39,10 +41,7 @@ function updateNames(data, categoricalNames) {
         renamedDimensions.forEach((dimension, index) => {
             const nameMap = renamedDimensions[index];
             let name = item.categories[index];
-            let newName = nameMap[name];
-            if (newName !== undefined) {
-                name = newName;
-            }
+            name = getCategoryValue(nameMap, name);
             names[index] = name;
             item.name = names;
         });
@@ -185,6 +184,17 @@ class DistributionGroup extends React.PureComponent {
         };
     }
 
+    onChartTypeChange = (event) => {
+        this.props.onDistributionPlotOptions({chartType: event.target.value});
+    };
+
+    onViolinScaleChange = (event) => {
+        this.props.onDistributionPlotOptions({violinScale: event.target.value});
+    };
+
+    onViolinShowBoxplot = (event) => {
+        this.props.onDistributionPlotOptions({violinShowBoxplot: event.target.checked});
+    };
     onSortOrderChanged = (event) => {
         this.props.onDistributionPlotOptions({sortBy: event.target.value});
     };
@@ -218,6 +228,7 @@ class DistributionGroup extends React.PureComponent {
         if (distributionData == null || distributionData.length === 0) {
             return null;
         }
+        const chartType = distributionPlotOptions.chartType;
         const meanAndPercentRange = getMeanAndPercentRange(distributionData);
         const meanRange = interpolator.scale !== INTERPOLATOR_SCALING_NONE ? [0, 1] : meanAndPercentRange.mean;
         const percentRange = meanAndPercentRange.percent;
@@ -259,7 +270,7 @@ class DistributionGroup extends React.PureComponent {
                 distributionPlotOptions.sortBy = distributionData[0].dimension;
             }
         }
-        const chartType = distributionPlotOptions.chartType;
+
         const maxRadius = 9;
         const minRadius = 1;
         const colorScale = createColorScale(interpolator).domain(meanRange);
@@ -274,6 +285,8 @@ class DistributionGroup extends React.PureComponent {
             return null;
         }
         const features = (data2d ? data2d[0] : selectedData2d[0]).map(item => item.feature);
+
+
         if (chartType === 'violin') {
             const allData = selectedData ? distributionData.concat(selectedData) : distributionData;
             features.forEach((feature) => {
@@ -352,9 +365,10 @@ class DistributionGroup extends React.PureComponent {
                                      onMaxUIChange={this.onMaxUIChange}
                 />}
                 {chartType !== 'violin' && <FormControl className={this.props.classes.formControl}>
-                    <InputLabel shrink={true}>Standardize</InputLabel>
+                    <InputLabel>Standardize</InputLabel>
                     <Select
-                        input={<Input size={"small"}/>}
+                        label={"Standardize"}
+                        size={"small"}
                         onChange={event => this.props.onColorScalingChange(event.target.value)}
                         value={interpolator.scale}
                     >
@@ -373,7 +387,8 @@ class DistributionGroup extends React.PureComponent {
                 <FormControl className={this.props.classes.formControl}>
                     <InputLabel shrink={true}>Sort By</InputLabel>
                     <Select
-                        input={<Input size={"small"}/>}
+                        label={"Sort By"}
+                        size={"small"}
                         onChange={this.onSortOrderChanged}
                         value={distributionPlotOptions.sortBy}
                     >
@@ -382,6 +397,54 @@ class DistributionGroup extends React.PureComponent {
                         ))}
                     </Select>
                 </FormControl>
+
+
+                {chartType === 'violin' && <FormControl className={this.props.classes.formControl}>
+                    <InputLabel id="violin-scale-label">Scale</InputLabel>
+                    <Select
+                        label={"Scale"}
+                        size={"small"}
+                        className={this.props.classes.select}
+                        labelId="violin-scale-label"
+                        value={distributionPlotOptions.violinScale}
+                        onChange={this.onViolinScaleChange}
+                    >
+                        <MenuItem value={'area'}>Area</MenuItem>
+                        <MenuItem value={'width'}>Width</MenuItem>
+                    </Select>
+                    <FormHelperText>If "area", violins have the same area. If "width", violins have the
+                        same
+                        maximum
+                        width.</FormHelperText>
+                </FormControl>}
+
+                {chartType === 'violin' && <div><FormControlLabel
+                    control={
+                        <Switch
+                            value={"violinShowBoxplot"}
+                            checked={distributionPlotOptions.violinShowBoxplot}
+                            onChange={this.onViolinShowBoxplot}
+                        />
+                    }
+                    label="Show Box Plot"
+                /></div>}
+
+                <FormControl className={this.props.classes.formControl}>
+                    <InputLabel id="dist-chart-type-label">Chart Type</InputLabel>
+                    <Select
+                        label={"Chart Type"}
+                        size={"small"}
+                        className={this.props.classes.select}
+                        labelId="dist-chart-type-label"
+                        value={chartType}
+                        onChange={this.onChartTypeChange}
+                    >
+                        {this.props.showDotPlotOption && <MenuItem value={'dotplot'}>Dot Plot</MenuItem>}
+                        <MenuItem value={'heatmap'}>Heatmap</MenuItem>
+                        <MenuItem value={'violin'}>Violin</MenuItem>
+                    </Select>
+                </FormControl>
+
             </Box>
         );
     }
