@@ -1,10 +1,10 @@
-import {Tooltip} from '@material-ui/core';
-import IconButton from '@material-ui/core/IconButton';
-import Menu from '@material-ui/core/Menu';
-import MenuItem from '@material-ui/core/MenuItem';
-import Typography from '@material-ui/core/Typography';
-import CloudDownloadIcon from '@material-ui/icons/CloudDownload';
-import PhotoCameraIcon from '@material-ui/icons/PhotoCamera';
+import {Tooltip} from '@mui/material';
+import IconButton from '@mui/material/IconButton';
+import Menu from '@mui/material/Menu';
+import MenuItem from '@mui/material/MenuItem';
+import Typography from '@mui/material/Typography';
+import CloudDownloadIcon from '@mui/icons-material/CloudDownload';
+import PhotoCameraIcon from '@mui/icons-material/PhotoCamera';
 import {cumsum} from 'd3-array';
 import {scaleLinear} from 'd3-scale';
 import React from 'react';
