@@ -1,5 +1,5 @@
-import Dialog from '@material-ui/core/Dialog';
-import DialogContent from '@material-ui/core/DialogContent';
+import Dialog from '@mui/material/Dialog';
+import DialogContent from '@mui/material/DialogContent';
 import React from 'react';
 import {connect} from 'react-redux';
 import {setDialog} from './actions';
