@@ -1,7 +1,7 @@
-import {makeStyles, ScatterPlot, ScatterPlotVisualizerSprites, ScatterPlotVisualizerSvgLabels} from 'scatter-gl';
+import {makeStyles, ScatterPlot, ScatterPlotVisualizerSprites, ScatterPlotVisualizerSvgLabels} from './scatter-gl';
 import {Color, OrthographicCamera, Vector3} from 'three';
 import {getEmbeddingKey} from './actions';
-import {getVisualizer} from './ScatterChartThree';
+import {getVisualizer, setAxesColors} from './ScatterChartThree';
 import {indexSort, randomSeq, rankIndexArray} from './util';
 
 export const POINT_VISUALIZER_ID = 'SPRITES';
@@ -115,7 +115,7 @@ export function getPositions(trace) {
     const halfCube = SCATTER_PLOT_CUBE_LENGTH / 2;
     const makeScaleRange = (range, base) => [
         -base * (range / maxRange),
-        base * (range / maxRange),
+        base * (range / maxRange)
     ];
     const xScale = makeScaleRange(xRange, halfCube);
     const yScale = makeScaleRange(yRange, halfCube);
@@ -214,7 +214,7 @@ export function getCategoryLabelsPositions(embedding, obsKeys, cachedData) {
         const halfCube = SCATTER_PLOT_CUBE_LENGTH / 2;
         const makeScaleRange = (range, base) => [
             -base * (range / maxRange),
-            base * (range / maxRange),
+            base * (range / maxRange)
         ];
         xScale = makeScaleRange(xRange, halfCube);
         yScale = makeScaleRange(yRange, halfCube);
@@ -250,39 +250,43 @@ export function getCategoryLabelsPositions(embedding, obsKeys, cachedData) {
     return {labels: labelValues, positions: labelPositions};
 }
 
-export function updateScatterChart(scatterPlot, traceInfo, selection, markerOpacity, unselectedMarkerOpacity, pointSize, categoricalNames = {}, chartOptions, obsCatKeys, cachedData) {
+export function updateScatterChart(scatterPlot, traceInfo, selection, markerOpacity, unselectedMarkerOpacity, pointSize, unselectedPointSize, categoricalNames = {}, chartOptions, obsCatKeys, cachedData, cameraDef) {
     const is3d = traceInfo.z != null;
     const colors = traceInfo.colors;
     let positions = traceInfo.positions;
 
-    const npoints = traceInfo.npoints;
+    const npoints = traceInfo.x.length;
     const isSelectionEmpty = selection.size === 0;
     const updateZ = !isSelectionEmpty && !is3d;
     if (updateZ) {
         positions = positions.slice();
     }
+    const scale = new Float32Array(traceInfo.x.length);
+    scale.fill(pointSize);
     for (let i = 0, j = 3, k = 2; i < npoints; i++, j += 4, k += 3) {
         const isSelected = isSelectionEmpty || selection.has(i);
         colors[j] = isSelected ? markerOpacity : unselectedMarkerOpacity;
+        if (!isSelected) {
+            scale[i] = unselectedPointSize;
+        }
         if (updateZ && !isSelected) {
             positions[k] = -1;
         }
     }
     scatterPlot.scene.background = chartOptions.darkMode ? new Color("rgb(0, 0, 0)") : null;
     scatterPlot.setDimensions(traceInfo.dimensions);
+    if (cameraDef) {
+        scatterPlot.updateFromCameraDef(cameraDef);
+    }
     let spriteVisualizer = getVisualizer(scatterPlot, POINT_VISUALIZER_ID);
     spriteVisualizer.styles.fog.enabled = chartOptions.showFog;
     const axes = scatterPlot.scene.getObjectByName('axes');
     if (axes) {
+        setAxesColors(scatterPlot, chartOptions.darkMode);
         axes.visible = chartOptions.showAxis;
     }
     scatterPlot.setPointColors(colors);
     scatterPlot.setPointPositions(positions);
-
-    // const {scaleDefault, scaleSelected, scaleHover} = scatterPlot.styles.point;
-
-    const scale = new Float32Array(traceInfo.npoints);
-    scale.fill(pointSize);
     scatterPlot.setPointScaleFactors(scale);
 
     const showLabels = obsCatKeys.length > 0;
@@ -317,7 +321,7 @@ export function getLabels(obsCat, labels, categoricalNames) {
         let value = [];
         for (let j = 0; j < array.length; j++) {
             let renamedValue = renamedCategories[j][array[j]];
-            value.push(renamedValue != null ? renamedValue : array[j]);
+            value.push(renamedValue != null && renamedValue.newValue != null ? renamedValue : array[j]);
         }
         labelStrings.push(value.join(','));
     }
