@@ -82,19 +82,22 @@ export function getTypeToMeasures(measures) {
     return typeToMeasures;
 }
 
-export function getBasis(basis, nbins = null, agg = null, dimensions = 2, precomputed = false) {
+export function getBasis(basis, dimensions = 2, mode = null) {
     dimensions = parseInt(dimensions);
     let coordinate_columns = [];
     for (let i = 0; i < dimensions; i++) {
         coordinate_columns.push(basis + '_' + i + 1);
     }
     let full_name = basis + '_' + dimensions;
-    if (nbins != null) {
-        full_name = full_name + '_' + nbins + '_' + agg;
+    if (mode != null) {
+        full_name = full_name + '_' + mode;
     }
     return {
-        'name': basis, 'dimensions': dimensions, 'coordinate_columns': coordinate_columns, 'nbins': nbins,
-        'agg': agg, 'full_name': full_name, 'precomputed': precomputed
+        name: basis,
+        dimensions: dimensions,
+        coordinate_columns: coordinate_columns,
+        mode: mode,
+        full_name: full_name
     };
 }
 
@@ -108,13 +111,9 @@ export function splitDataFilter(data_filter) {
             let user_filter = user_filters[i];
             let key = user_filter[0];
             if (isObject(key)) {
-                let basis = getBasis(key.basis, key.nbins, key.agg,
-                    key.ndim || 2, key.precomputed);
-                basis_list.push(basis);
+                basis_list.push(key);
             } else {
-
                 const {name, type} = getVarNameType(key);
-
                 user_filter[0] = name;
                 if (type === 'X') {
                     var_keys.add(name);
@@ -138,33 +137,40 @@ export function computeDerivedStats(result, q, cachedData) {
     }
 
     if (q.groupedStats) {
-        const dimensions = q.groupedStats.dimensions || []; // array of arrays
+        const dimensions = q.groupedStats.dimensions || []; // array of arrays to support multiple groupings in the future
         const measures = q.groupedStats.measures || [];
         const typeToMeasures = getTypeToMeasures(measures);
-        if (dimensions.length > 0 && typeToMeasures.X.length > 0) {
+        if (dimensions.length > 0) {
             // TODO, currently we only handle dimensions[0]
-            result.distribution = groupedStats(getVectors(cachedData, dimensions[0]), getVectors(cachedData, typeToMeasures.X));
+            const groupDimensionInfo = groupDimensions(getVectors(cachedData, dimensions[0]));
+            const distribution = {};
+            for (const key in typeToMeasures) {
+                const measures = typeToMeasures[key];
+                if (measures.length > 0) {
+                    distribution[key] = groupedStats(groupDimensionInfo, getVectors(cachedData, measures));
+                }
+            }
+            result.distribution = distribution;
         }
     }
 
     if (q.selection) {
         const dimensions = q.selection.dimensions || [];
         const measures = q.selection.measures || [];
-        // const embeddings = q.selection.embeddings || [];
         const typeToMeasures = getTypeToMeasures(measures);
         result.selection = {};
         result.selection.indices = getPassingFilterIndices(cachedData, q.selection.filter);
         const selectedIndices = Array.from(result.selection.indices);
-        // if (embeddings.length > 0) {
-        //     result.selection.coordinates = {};
-        //     embeddings.forEach(embedding => {
-        //         let basis = getBasis(embedding.basis, embedding.nbins,
-        //             embedding.agg, embedding.ndim || 2, embedding.precomputed);
-        //         result.selection.coordinates[basis.full_name] = {'indices_or_bins': selectedIndices};
-        //     });
-        // }
-        if (dimensions.length > 0 && typeToMeasures.X.length > 0) {
-            result.selection.distribution = groupedStats(getVectors(cachedData, dimensions, selectedIndices), getVectors(cachedData, typeToMeasures.X, selectedIndices));
+        if (dimensions.length > 0) {
+            const groupDimensionInfo = groupDimensions(getVectors(cachedData, dimensions, selectedIndices));
+            const distribution = {};
+            for (const key in typeToMeasures) {
+                const measures = typeToMeasures[key];
+                if (measures.length > 0) {
+                    distribution[key] = groupedStats(groupDimensionInfo, getVectors(cachedData, measures, selectedIndices));
+                }
+            }
+            result.selection.distribution = distribution;
         }
         result.selection.summary = getStats(getVectors(cachedData, dimensions, selectedIndices), getVectors(cachedData, typeToMeasures.obs, result.selection.indices),
             getVectors(cachedData, typeToMeasures.X, selectedIndices));
@@ -186,8 +192,8 @@ export function valueCounts(v) {
 }
 
 
-export function groupedStats(dimensions, varMeasures) {
-    const categoryMap = {};
+export function groupDimensions(dimensions) {
+    const keyToIndices = {};
     const ndim = dimensions.length;
     const size = dimensions[0].size();
     const tmp = [];
@@ -196,42 +202,32 @@ export function groupedStats(dimensions, varMeasures) {
             tmp[j] = dimensions[j].get(i);
         }
         const key = tmp.join(', ');
-        let value = categoryMap[key];
+        let value = keyToIndices[key];
         if (value === undefined) {
             value = {key: tmp.slice(), indices: []};
-            categoryMap[key] = value;
+            keyToIndices[key] = value;
         }
         value.indices.push(i);
     }
-    let categories = Object.keys(categoryMap);
+    let categories = Object.keys(keyToIndices);
     const dimensionNames = [];
     for (let j = 0; j < ndim; j++) {
         dimensionNames[j] = dimensions[j].getName();
     }
     const dimensionName = dimensionNames.join('-');
-    // each entry {dimension:dimensionName, name:category, feature:'', mean:0, percentExpressed:xx}
-    let result = [];
-    categories.forEach(category => {
-        const value = categoryMap[category];
-        varMeasures.forEach((v) => {
+    return {categories, keyToIndices, dimensionName, dimensionNames};
+}
 
-            // let otherIndices = [];
-            // categories.forEach(otherCategory => {
-            //     if (category !== otherCategory) {
-            //         otherIndices = otherIndices.concat(categoryToIndices[otherCategory]);
-            //     }
-            // });
-            // const restVector = new SlicedVector(v, otherIndices);
-            // const restStats = stats(restVector);
-            // results.values[index].restMean.push(restStats.mean);
-            // results.values[index].restFractionExpressed.push(restStats.numExpressed / restVector.size());
+export function groupedStats(groupDimensionInfo, vectors) {
+    let result = [];
+    groupDimensionInfo.categories.forEach(category => {
+        const value = groupDimensionInfo.keyToIndices[category];
+        vectors.forEach((v) => {
             const categoryVector = new SlicedVector(v, value.indices);
             const categoryStats = stats(categoryVector);
-            // results.values[index].mean.push(categoryStats.mean);
-            // results.values[index].percentExpressed.push(categoryStats.numExpressed / categoryVector.size());
             const entry = {
-                dimension: dimensionName,
-                dimensions: dimensionNames,
+                dimension: groupDimensionInfo.dimensionName,
+                dimensions: groupDimensionInfo.dimensionNames,
                 categories: value.key,
                 feature: v.getName(),
                 mean: categoryStats.mean,
@@ -239,7 +235,6 @@ export function groupedStats(dimensions, varMeasures) {
                 percentExpressed: 100 * (categoryStats.numExpressed / categoryVector.size()),
                 vector: categoryVector
             };
-
             result.push(entry);
         });
     });
