@@ -3,43 +3,51 @@ import {render} from 'react-dom';
 import {Provider} from 'react-redux';
 import {applyMiddleware, createStore} from 'redux';
 import thunkMiddleware from 'redux-thunk';
-import {initGapi} from './actions';
-import App from './App';
+import {initGapi, SET_DATASET, SET_EMAIL, SET_SERVER_INFO} from './actions';
 import rootReducer from './reducers';
+import AppWrapper from './AppWrapper';
 // import * as serviceWorker from './serviceWorker';
+import mixpanel from 'mixpanel-browser';
+
+let useMixPanel = false;
 
 
 const logger = store => next => action => {
-    console.log(action);
-    let result = next(action);
-    return result;
+    if (action.type === SET_SERVER_INFO) {
+        if (action.payload.mixpanel) {
+            mixpanel.init(action.payload.mixpanel);
+            useMixPanel = true;
+        }
+    }
+    if (useMixPanel) {
+        if (action.type === SET_DATASET) {
+            mixpanel.track('Open Dataset', {name: action.payload.name, id: action.payload.id});
+        } else if (action.type === SET_EMAIL) {
+            mixpanel.identify(action.payload);
+        }
+    }
+    return next(action);
 };
 const store = createStore(
     rootReducer,
     applyMiddleware(
-        thunkMiddleware
-    ),
+        thunkMiddleware, logger
+    )
 );
 
 function main() {
     render(
         <Provider store={store}>
             {/*<React.StrictMode>*/}
-            <App/>
+
+            <AppWrapper/>
+
             {/*</React.StrictMode>*/}
         </Provider>,
-        document.getElementById('root'),
+        document.getElementById('root')
     );
 }
 
-// if (module.hot) {
-//   if (process.env.NODE_ENV !== 'production' && module.hot) {
-//     module.hot.accept('./reducers', () => store.replaceReducer(rootReducer));
-//   }
-//   if (process.env.NODE_ENV !== 'production' && module.hot) {
-//     module.hot.accept('./App', main);
-//   }
-// }
 store.dispatch(initGapi());
 main();
 
