@@ -2,11 +2,20 @@ import {isPlainObject} from 'lodash';
 import {combineReducers} from 'redux';
 import {
     ADD_DATASET,
+    DEFAULT_DARK_MODE,
+    DEFAULT_DISTRIBUTION_PLOT_INTERPOLATOR,
+    DEFAULT_DRAG_MODE,
+    DEFAULT_INTERPOLATOR,
+    DEFAULT_LABEL_FONT_SIZE,
+    DEFAULT_LABEL_STROKE_WIDTH,
+    DEFAULT_MARKER_OPACITY,
+    DEFAULT_POINT_SIZE,
+    DEFAULT_SHOW_AXIS,
+    DEFAULT_SHOW_FOG,
+    DEFAULT_UNSELECTED_MARKER_OPACITY,
     DELETE_DATASET,
-    getTraceKey,
     RESTORE_VIEW,
     SET_ACTIVE_FEATURE,
-    SET_CATEGORICAL_COLOR,
     SET_CATEGORICAL_NAME,
     SET_CHART_OPTIONS,
     SET_CHART_SIZE,
@@ -15,6 +24,7 @@ import {
     SET_DATASET_CHOICES,
     SET_DATASET_FILTER,
     SET_DATASET_FILTERS,
+    SET_DATASET_VIEWS,
     SET_DIALOG,
     SET_DISTRIBUTION_DATA,
     SET_DISTRIBUTION_PLOT_INTERPOLATOR,
@@ -28,6 +38,7 @@ import {
     SET_INTERPOLATOR,
     SET_JOB_RESULT,
     SET_JOB_RESULTS,
+    SET_LEGEND_SCROLL_POSITION,
     SET_LOADING,
     SET_LOADING_APP,
     SET_MARKER_OPACITY,
@@ -43,8 +54,11 @@ import {
     SET_SERVER_INFO,
     SET_TAB,
     SET_UNSELECTED_MARKER_OPACITY,
+    SET_UNSELECTED_POINT_SIZE,
     SET_USER,
-    UPDATE_DATASET,
+    UPDATE_CATEGORICAL_COLOR,
+    UPDATE_CATEGORICAL_NAME,
+    UPDATE_DATASET
 } from '../actions';
 import {createCategoryToStats} from '../MetaEmbedding';
 import {
@@ -56,29 +70,21 @@ import {
     TRACE_TYPE_META_IMAGE,
     updateTraceColors
 } from '../util';
+import {updateJob} from '../DotPlotJobResultsPanel';
 
-export const DEFAULT_BIN_SUMMARY = 'max';
-export const DEFAULT_NUMBER_BINS = 500;
-export const DEFAULT_POINT_SIZE = 1;
-export const DEFAULT_MARKER_OPACITY = 1;
-export const DEFAULT_UNSELECTED_MARKER_OPACITY = 0.1;
-export const DEFAULT_INTERPOLATOR = 'Viridis';
-export const DEFAULT_DISTRIBUTION_PLOT_INTERPOLATOR = 'Reds';
-export const DEFAULT_DRAG_MODE = 'pan';
-export const DEFAULT_SHOW_LABELS = false;
-export const DEFAULT_SHOW_AXIS = true;
-export const DEFAULT_SHOW_FOG = false;
-export const DEFAULT_DARK_MODE = false;
-export const DEFAULT_LABEL_FONT_SIZE = 14;
-export const DEFAULT_LABEL_STROKE_WIDTH = 4;
 
-const DEFAULT_DIST_PLOT_OPTIONS = {
+const DIST_PLOT_OPTIONS = {
     chartType: 'dotplot',
     violinScale: 'width',
     violinHeight: 100,
     violinWidth: 80,
     violinShowBoxplot: true
 };
+const DEFAULT_DIST_PLOT_OPTIONS = {
+    X: DIST_PLOT_OPTIONS,
+    modules: Object.assign({}, DIST_PLOT_OPTIONS, {chartType: 'heatmap'}),
+    obs: Object.assign({}, DIST_PLOT_OPTIONS, {chartType: 'violin'})
+};
 
 const DEFAULT_INTERPOLATOR_OBJ = {
     name: DEFAULT_INTERPOLATOR,
@@ -86,14 +92,18 @@ const DEFAULT_INTERPOLATOR_OBJ = {
     reversed: false
 };
 
-const DEFAULT_DISTRIBUTION_PLOT_INTERPOLATOR_OBJ = {
+const DISTRIBUTION_PLOT_INTERPOLATOR_OBJ = {
     name: DEFAULT_DISTRIBUTION_PLOT_INTERPOLATOR,
     value: getInterpolator(DEFAULT_DISTRIBUTION_PLOT_INTERPOLATOR),
     reversed: false,
     scale: INTERPOLATOR_SCALING_NONE
 };
 
-
+const DEFAULT_DISTRIBUTION_PLOT_INTERPOLATOR_OBJ = {
+    X: DISTRIBUTION_PLOT_INTERPOLATOR_OBJ,
+    modules: DISTRIBUTION_PLOT_INTERPOLATOR_OBJ,
+    obs: DISTRIBUTION_PLOT_INTERPOLATOR_OBJ
+};
 const DEFAULT_PRIMARY_CHART_SIZE = {
     width: window.innerWidth - 280,
     height: Math.max(300, window.innerHeight - 370)
@@ -161,6 +171,7 @@ function embeddingLabels(state = [], action) {
     }
 }
 
+
 function chartOptions(state = DEFAULT_CHART_OPTIONS, action) {
     switch (action.type) {
         case SET_CHART_OPTIONS:
@@ -226,7 +237,7 @@ function datasetChoices(state = [], action) {
     }
 }
 
-// set the selected embeddings, each embedding has name (str) e.g X_umap, nbins (int), _nbins (str), agg (str), bin (boolean), dimensions (int), precomputed (bool)
+// set the selected embeddings, each embedding has name (str) e.g X_umap, dimensions (int), mode (str)
 function embeddings(state = [], action) {
     switch (action.type) {
         case SET_SELECTED_EMBEDDING:
@@ -401,6 +412,10 @@ function globalFeatureSummary(state = {}, action) {
 function serverInfo(state = {}, action) {
     switch (action.type) {
         case SET_SERVER_INFO:
+            if (action.payload.ontology && action.payload.ontology.cellTypes) {
+                action.payload.ontology.cellTypes.forEach(item => item.text = item.name);
+                action.payload.ontology.cellTypes.sort((item1, item2) => NATSORT(item1.text, item2.text));
+            }
             return action.payload;
         default:
             return state;
@@ -480,11 +495,15 @@ function distributionPlotOptions(state = DEFAULT_DIST_PLOT_OPTIONS, action) {
     }
 }
 
+
 function jobResult(state = null, action) {
     switch (action.type) {
         case SET_DATASET:
             return null;
         case SET_JOB_RESULT:
+            if (action.payload != null && action.payload.type == 'de') {
+                updateJob(action.payload); // initialize
+            }
             return action.payload;
         default:
             return state;
@@ -503,27 +522,27 @@ function jobResults(state = [], action) {
     }
 }
 
-function distributionData(state = [], action) {
+function distributionData(state = {}, action) {
     switch (action.type) {
         case SET_CATEGORICAL_NAME:
-            return state.slice();
+            return Object.assign({}, state);
         case SET_DISTRIBUTION_DATA:
             return action.payload;
         case SET_DATASET:
-            return [];
+            return {};
         default:
             return state;
     }
 }
 
-function selectedDistributionData(state = [], action) {
+function selectedDistributionData(state = {}, action) {
     switch (action.type) {
         case SET_CATEGORICAL_NAME:
-            return state.slice();
+            return Object.assign({}, state);
         case SET_SELECTED_DISTRIBUTION_DATA:
             return action.payload;
         case SET_DATASET:
-            return [];
+            return {};
         default:
             return state;
     }
@@ -535,13 +554,22 @@ function datasetFilters(state = [], action) {
         case SET_DATASET:
             return [];
         case SET_DATASET_FILTERS:
-            //action.payload.sort()
             return action.payload;
         default:
             return state;
     }
 }
 
+function datasetViews(state = [], action) {
+    switch (action.type) {
+        case SET_DATASET:
+            return [];
+        case SET_DATASET_VIEWS:
+            return action.payload;
+        default:
+            return state;
+    }
+}
 
 function datasetFilter(state = {}, action) {
     switch (action.type) {
@@ -569,20 +597,19 @@ function combineDatasetFilters(state = 'and', action) {
     }
 }
 
-// category -> value -> newValue
+
+// category -> originalValue -> {newValue, positiveMarkers, negativeMarkers, color}
 function categoricalNames(state = {}, action) {
     switch (action.type) {
         case SET_CATEGORICAL_NAME:
+            return action.payload;
+        case UPDATE_CATEGORICAL_NAME:
             let category = state[action.payload.name];
             if (category === undefined) {
                 category = {};
                 state[action.payload.name] = category;
             }
-            if (action.payload.value == null || action.payload.value === '') {
-                delete category[action.payload.oldValue];
-            } else {
-                category[action.payload.oldValue] = action.payload.value;
-            }
+            category[action.payload.originalValue] = action.payload;
             return Object.assign({}, state);
         case SET_DATASET:
             return {};
@@ -596,41 +623,18 @@ function cachedData(state = {}, action) {
     switch (action.type) {
         case SET_DATASET :
             return {};
+        default:
+            return state;
     }
-    return state;
+
 }
 
-function sortEmbeddingTraces(traces) {
-    // sort by feature insertion order, then embedding
-    const featureToMinIndex = {};
-    for (let i = 0; i < traces.length; i++) {
-        let prior = featureToMinIndex[traces[i].name];
-        let index = i;
-        if (prior !== undefined) {
-            index = Math.min(index, prior);
-        }
-        featureToMinIndex[traces[i].name] = index;
-    }
-    traces.sort((a, b) => {
-        if (a.name === b.name) {
-            const val = NATSORT(a.embedding.name, b.embedding.name);
-            if (val !== 0) {
-                return val;
-            }
-        }
-        // sortIndex added if user drags and drops gallery chart
-        const index1 = a.sortIndex !== undefined ? a.sortIndex : featureToMinIndex[a.name];
-        const index2 = b.sortIndex !== undefined ? b.sortIndex : featureToMinIndex[b.name];
-        return index1 - index2;
-    });
-    return traces;
-}
 
 // each item has  data (list of traces, each trace has x, y, etc.), layout
 function embeddingData(state = [], action) {
     switch (action.type) {
         case SET_EMBEDDING_DATA :
-            return sortEmbeddingTraces(action.payload);
+            return action.payload;
         case SET_SELECTION:
             state.forEach(trace => {
                 if (trace.type === TRACE_TYPE_META_IMAGE) {
@@ -666,12 +670,11 @@ function embeddingData(state = [], action) {
                 }
             });
             return state.slice();
-        case SET_CATEGORICAL_COLOR:
+        case UPDATE_CATEGORICAL_COLOR:
             state.forEach((trace) => {
                 if (!trace.continuous && trace.name === action.payload.name) {
-                    let index = trace.colorScale.domain().indexOf(action.payload.value);
-                    let range = trace.colorScale.range();
-                    range[index] = action.payload.color;
+                    const range = trace.colorScale.range();
+                    range[trace.colorScale.domain().indexOf(action.payload.originalValue)] = action.payload.color;
                     trace.colorScale.range(range);
                     updateTraceColors(trace);
                 }
@@ -706,6 +709,17 @@ export function pointSize(state = DEFAULT_POINT_SIZE, action) {
     }
 }
 
+export function unselectedPointSize(state = DEFAULT_POINT_SIZE, action) {
+    switch (action.type) {
+        case SET_UNSELECTED_POINT_SIZE:
+            return action.payload;
+        case RESTORE_VIEW:
+            return action.payload.unselectedPointSize != null ? action.payload.unselectedPointSize : state;
+        default:
+            return state;
+    }
+}
+
 
 // used to restore state when toggling datasets
 export function savedDatasetState(state = {}, action) {
@@ -717,6 +731,7 @@ export function savedDatasetState(state = {}, action) {
     }
 }
 
+
 // object with name, type, embeddingKey
 export function activeFeature(state = {}, action) {
     switch (action.type) {
@@ -724,14 +739,19 @@ export function activeFeature(state = {}, action) {
             return null;
         case SET_ACTIVE_FEATURE:
             return action.payload;
-        case SET_EMBEDDING_DATA:
-            let traces = action.payload.filter(trace => trace.active);
-            if (traces.length === 0) {
-                return null;
-            }
-            const trace = traces[traces.length - 1];
-            const embeddingKey = getTraceKey(trace); // last feature becomes primary
-            return {name: trace.name, type: trace.featureType, embeddingKey: embeddingKey};
+        default:
+            return state;
+    }
+}
+
+// feature name (e.g. leiden) -> scroll position for restoring state
+export function legendScrollPosition(state = {}, action) {
+    switch (action.type) {
+        case SET_DATASET:
+            return {};
+        case SET_LEGEND_SCROLL_POSITION:
+            state[action.payload.name] = action.payload.value;
+            return state;
         default:
             return state;
     }
@@ -769,6 +789,7 @@ export default combineReducers({
     datasetChoices,
     datasetFilter,
     datasetFilters,
+    datasetViews,
     dialog,
     distributionData,
     distributionPlotOptions,
@@ -782,6 +803,7 @@ export default combineReducers({
     interpolator,
     jobResult,
     jobResults,
+    legendScrollPosition,
     loading,
     loadingApp,
     markerOpacity,
@@ -796,5 +818,6 @@ export default combineReducers({
     serverInfo,
     tab,
     unselectedMarkerOpacity,
+    unselectedPointSize,
     user
 });
