@@ -1,13 +1,14 @@
-import Link from '@material-ui/core/Link';
-import {withStyles} from '@material-ui/core/styles';
-import Typography from '@material-ui/core/Typography';
-import {shuffle} from 'd3-array';
+import Link from '@mui/material/Link';
+import withStyles from '@mui/styles/withStyles';
+import Typography from '@mui/material/Typography';
+import {extent, shuffle} from 'd3-array';
 import {color} from 'd3-color';
 import {scaleLinear, scaleSequential} from 'd3-scale';
 import * as scaleChromatic from 'd3-scale-chromatic';
 import natsort from 'natsort';
 import React from 'react';
 import simplify from 'simplify-js';
+import {isString} from "lodash";
 
 export const NATSORT = natsort({insensitive: true});
 export const interpolators = {};
@@ -60,6 +61,15 @@ interpolators['Cyclical'] = ['interpolateRainbow', 'interpolateSinebow'];
 //     '#c5b0d5', '#8c564b', '#c49c94', '#e377c2', '#f7b6d2', '#7f7f7f',
 //     '#c7c7c7', '#bcbd22', '#dbdb8d', '#17becf', '#9edae5'];
 
+
+export const SERVER_CAPABILITY_RENAME_CATEGORIES = 'SERVER_CAPABILITY_RENAME_CATEGORIES';
+export const SERVER_CAPABILITY_JOBS = 'SERVER_CAPABILITY_JOBS';
+export const SERVER_CAPABILITY_FEATURE_SETS = 'SERVER_CAPABILITY_FEATURE_SETS';
+export const SERVER_CAPABILITY_LINKS = 'SERVER_CAPABILITY_LINKS';
+export const SERVER_CAPABILITY_EDIT_DATASET = 'SERVER_CAPABILITY_EDIT_DATASET';
+export const SERVER_CAPABILITY_ADD_DATASET = 'SERVER_CAPABILITY_ADD_DATASET';
+export const SERVER_CAPABILITY_DELETE_DATASET = 'SERVER_CAPABILITY_DELETE_DATASET';
+
 export const CATEGORY_20B = [
     '#393b79', '#5254a3', '#6b6ecf',
     '#9c9ede', '#637939', '#8ca252', '#b5cf6b', '#cedb9c', '#8c6d31',
@@ -76,7 +86,7 @@ export const FEATURE_TYPE = {
     OBS_CAT: 'obsCat',
     X: 'X',
     FEATURE_SET: 'featureSet',
-    METAFEATURE: 'metafeature',
+    MODULE: 'module',
     COUNT: 'count'
 };
 
@@ -95,14 +105,22 @@ export const INTERPOLATOR_SCALING_MIN_MAX_FEATURE = 'min_max_feature';
 export const INTERPOLATOR_SCALING_MIN_MAX_CATEGORY = 'min_max_category';
 export const INTERPOLATOR_SCALING_NONE = 'none';
 
+export function getCategoryValue(renamedCategories, category) {
+    let renamedCategoryValue = renamedCategories[category];
+    if (renamedCategoryValue != null && renamedCategoryValue.newValue != null) {
+        return renamedCategoryValue.newValue;
+    }
+    return category;
+}
+
 export function isMac() {
     return window.navigator.platform.toLowerCase().indexOf('mac') !== -1;
 }
 
 const reactMarkdownStyles = (theme) => ({
     listItem: {
-        marginTop: theme.spacing(1),
-    },
+        marginTop: theme.spacing(1)
+    }
 });
 
 
@@ -111,22 +129,22 @@ export const REACT_MD_OVERRIDES = {
         component: Typography,
         props: {
             gutterBottom: true,
-            variant: 'h5',
-        },
+            variant: 'h5'
+        }
     },
     h2: {component: Typography, props: {gutterBottom: true, variant: 'h6'}},
     h3: {component: Typography, props: {gutterBottom: true, variant: 'subtitle1'}},
     h4: {
         component: Typography,
-        props: {gutterBottom: true, variant: 'caption', paragraph: true},
+        props: {gutterBottom: true, variant: 'caption', paragraph: true}
     },
     h5: {
         component: Typography,
-        props: {gutterBottom: true, variant: 'caption', paragraph: true},
+        props: {gutterBottom: true, variant: 'caption', paragraph: true}
     },
     h6: {
         component: Typography,
-        props: {gutterBottom: true, variant: 'caption', paragraph: true},
+        props: {gutterBottom: true, variant: 'caption', paragraph: true}
     },
     p: {component: Typography, props: {paragraph: true}},
     a: {component: Link, props: {target: '_blank'}},
@@ -135,10 +153,85 @@ export const REACT_MD_OVERRIDES = {
             <li className={classes.listItem}>
                 <Typography component="span" {...props} />
             </li>
-        )),
+        ))
     }
 };
 
+export function summarizeDensity(values, index, selection, summarizationMethod) {
+    let newValues = [];
+    let summarize;
+
+    if (summarizationMethod === 'max') {
+        summarize = (indices) => {
+            let max = -Number.MAX_VALUE;
+            for (let j = 0, nindices = indices.length; j < nindices; j++) {
+                max = Math.max(values[indices[j]], max);
+            }
+            return max;
+        };
+    } else if (summarizationMethod === 'mode') {
+        summarize = (indices) => {
+            let valueToCount = new Map();
+            for (let j = 0, nindices = indices.length; j < nindices; j++) {
+                const value = values[indices[j]];
+                let prior = valueToCount.get(value);
+                if (prior === undefined) {
+                    prior = 0;
+                }
+                valueToCount.set(value, 1 + prior);
+            }
+            // TODO compute purity
+            let max = -Number.MAX_VALUE;
+            let maxValue = null;
+            valueToCount.forEach((count, value) => {
+                if (count > max) {
+                    max = count;
+                    maxValue = value;
+                }
+            });
+            return maxValue;
+        };
+
+    } else {
+        throw new Error();
+    }
+    const isSelectionEmpty = selection.size === 0;
+    for (let i = 0, n = index.length; i < n; i++) {
+        const indices = index[i];
+        const summarizedValue = summarize(isSelectionEmpty ? indices : indices.filter(item => selection.has(item)));
+        newValues.push(summarizedValue);
+    }
+
+    return newValues;
+}
+
+export function createEmbeddingDensity(xcoords, ycoords) {
+    const nbins = xcoords.length > 18367 ? Math.floor(Math.sqrt(xcoords.length / 1000)) * 14 : 60;
+    const xscale = scaleLinear().domain(extent(xcoords)).range([0, nbins - 1]);
+    const yscale = scaleLinear().domain(extent(ycoords)).range([0, nbins - 1]);
+    const binMap = new Map();
+    for (let i = 0, n = xcoords.length; i < n; i++) {
+        const xcoordNew = Math.floor(xscale(xcoords[i]));
+        const ycoordNew = Math.floor(yscale(ycoords[i]));
+        const bin = xcoordNew * nbins + ycoordNew;
+        let binValue = binMap.get(bin);
+        if (binValue === undefined) {
+            binValue = {indices: [], x: xcoordNew, y: ycoordNew};
+            binMap.set(bin, binValue);
+        }
+        binValue.indices.push(i);
+    }
+    const xvalues = [];
+    const yvalues = [];
+    const newIndexToOriginalIndices = [];
+    binMap.forEach((binValue, bin) => {
+        xvalues.push(binValue.x);
+        yvalues.push(binValue.y);
+        newIndexToOriginalIndices.push(binValue.indices);
+    });
+    return {x: xvalues, y: yvalues, mode: 'density', nbins: nbins, index: newIndexToOriginalIndices};
+}
+
 export function scaleConstantRange(value) {
 
     function scale(x) {
@@ -158,6 +251,38 @@ export function scaleConstantRange(value) {
     return scale;
 }
 
+export function copyToClipboard(text) {
+    const container = document.activeElement;
+    const fakeElem = document.createElement('textarea');
+    const isRTL = document.documentElement.getAttribute('dir') == 'rtl';
+    fakeElem.style.fontSize = '12pt';
+    fakeElem.style.border = '0';
+    fakeElem.style.padding = '0';
+    fakeElem.style.margin = '0';
+    // Move element out of screen horizontally
+    fakeElem.style.position = 'absolute';
+    fakeElem.style[isRTL ? 'right' : 'left'] = '-9999px';
+    fakeElem.setAttribute('readonly', '');
+    // Move element to the same position vertically
+    let yPosition = window.pageYOffset || document.documentElement.scrollTop;
+    fakeElem.style.top = yPosition + 'px';
+    fakeElem.value = text;
+    container.appendChild(fakeElem);
+
+    fakeElem.select();
+    fakeElem.setSelectionRange(0, fakeElem.value.length);
+
+    document.execCommand('copy');
+
+    const fakeHandlerCallback = (event) => {
+        document.activeElement.blur();
+        window.getSelection().removeAllRanges();
+        container.removeChild(fakeElem);
+        container.removeEventListener('click', fakeHandlerCallback);
+    };
+    container.addEventListener('click', fakeHandlerCallback);
+}
+
 export function stripTrailingZeros(s) {
     let index = s.lastIndexOf('.');
     let ending = s.substring(index + 1);
@@ -230,18 +355,56 @@ function getColorsRgba(trace) {
     const RGBA_NUM_ELEMENTS = 4;
     const rgbScale = getRgbScale();
 
-    let dst = 0;
-    let colorScale = trace.colorScale;
-    const n = trace.x.length;
-    const colors = new Float32Array(n * RGBA_NUM_ELEMENTS);
 
-    for (let i = 0; i < n; ++i) {
-        let c = color(colorScale(trace.values[i]));
-        colors[dst++] = rgbScale(c.r);
-        colors[dst++] = rgbScale(c.g);
-        colors[dst++] = rgbScale(c.b);
-        colors[dst++] = 1;
+    let colorScale = trace.colorScale;
+    const npoints = trace.x.length;
+    const colors = new Float32Array(npoints * RGBA_NUM_ELEMENTS);
+
+    function rescaleRgb(c) {
+        c.r = rgbScale(c.r);
+        c.g = rgbScale(c.g);
+        c.b = rgbScale(c.b);
+        return c;
     }
+
+    if (!trace.continuous) {
+        const categories = colorScale.domain();
+        const category2color = new Map();
+        categories.forEach(category => {
+            category2color.set(category, rescaleRgb(color(colorScale(category))));
+        });
+        for (let i = 0, dst = 0; i < npoints; ++i) {
+            const c = category2color.get(trace.values[i]);
+            colors[dst++] = c.r;
+            colors[dst++] = c.g;
+            colors[dst++] = c.b;
+            colors[dst++] = 1;
+        }
+    } else {
+        const ncolors = 500;
+        const colorBins = [];
+        const domain = colorScale.domain();
+        const binScale = scaleLinear().domain(domain).range([0, ncolors - 1]);
+        for (let i = 0; i < ncolors; ++i) {
+            const value = binScale.invert(i);
+            colorBins.push(rescaleRgb(color(colorScale(value))));
+        }
+        for (let i = 0, dst = 0; i < npoints; ++i) {
+            const value = trace.values[i];
+            let bin = Math.ceil(binScale(value));
+            if (bin < 0) {
+                bin = 0;
+            } else if (bin >= ncolors) {
+                bin = ncolors - 1;
+            }
+            const c = colorBins[bin];
+            colors[dst++] = c.r;
+            colors[dst++] = c.g;
+            colors[dst++] = c.b;
+            colors[dst++] = 1;
+        }
+    }
+
     return colors;
 }
 
@@ -250,7 +413,7 @@ export function updateTraceColors(traceInfo) {
         let colors = [];
         let colorScale = traceInfo.colorScale;
         const colorMapper = rgb => rgb.formatHex();
-        for (let i = 0, n = traceInfo.npoints; i < n; i++) {
+        for (let i = 0, n = traceInfo.x.length; i < n; i++) {
             let rgb = color(colorScale(traceInfo.values[i]));
             colors.push(colorMapper(rgb));
         }
@@ -580,17 +743,23 @@ export function addFeatureSetsToX(featureSets, X) {
     });
 }
 
-export function getFeatureSets(markers, featureSetIds) {
-    let featureSets = [];
-    featureSetIds.forEach(featureSetId => {
-        for (let i = 0; i < markers.length; i++) {
-            if (markers[i].id === featureSetId) {
-                featureSets.push(markers[i]);
-                break;
+export function getFeatureSets(markers, featureSets) {
+    const filteredFeatureSets = [];
+    if (featureSets.length > 0) {
+        let markerIdToMarker = new Map();
+        markers.forEach(m => {
+            markerIdToMarker.set(m.id, m);
+        });
+
+        featureSets.forEach(featureSet => {
+            const featureSetId = isString(featureSet) ? featureSet : featureSet.value;
+            const m = markerIdToMarker.get(featureSetId);
+            if (m != null) {
+                filteredFeatureSets.push(m);
             }
-        }
-    });
-    return featureSets;
+        });
+    }
+    return filteredFeatureSets;
 }
 
 export function splitSearchTokens(tokens) {
@@ -598,8 +767,7 @@ export function splitSearchTokens(tokens) {
     let obs = [];
     let obsCat = [];
     let featureSets = [];
-    let featureSetsAdd = [];
-    let metafeatures = [];
+    let modules = [];
     tokens.forEach(token => {
         if (token.type === FEATURE_TYPE.X) {
             X.push(token.value);
@@ -609,11 +777,11 @@ export function splitSearchTokens(tokens) {
             obsCat.push(token.value);
         } else if (token.type === FEATURE_TYPE.FEATURE_SET) {
             featureSets.push(token.value);
-        } else if (token.type === FEATURE_TYPE.METAFEATURE) {
-            metafeatures.push(token.value);
+        } else if (token.type === FEATURE_TYPE.MODULE) {
+            modules.push(token.value);
         } else {
             console.log('Unknown type: ' + token);
         }
     });
-    return {X: X, obs: obs, obsCat: obsCat, featureSets: featureSets, metafeatures: metafeatures, featureSetsAdd: []};
+    return {X: X, obs: obs, obsCat: obsCat, featureSets: featureSets, modules: modules, featureSetsAdd: []};
 }
